@@ -15,7 +15,7 @@ class LED : public Fl_Group {
 
     int width, height;
     int x_origin, y_origin;
-    int state;
+    LED_STATE state;
 
     Fl_Color LED_color;
     Fl_Color bg_color;
