@@ -33,6 +33,7 @@
 #include "gettext.h"
 #include "config.h"
 
+#include "atom.h"
 #include "cfg.h"
 #include "butt.h"
 #include "util.h"
@@ -42,9 +43,13 @@
 #include "fl_funcs.h"
 #include "shoutcast.h"
 #include "icecast.h"
+#include "webrtc.h"
 #include "strfuncs.h"
 #include "fl_timer_funcs.h"
+#include "logos.h"
+#include "Fl_LED.h"
 #include "command.h"
+#include "url.h"
 #ifdef WITH_RADIOCO
 #include "radioco.h"
 #endif
@@ -57,18 +62,32 @@ const char *(*current_track_app)(int);
 
 void split_recording_file(void);
 
-pthread_t split_recording_file_thread;
-pthread_t request_listener_count_thread;
-static int g_listener_count = -1;
+pthread_t split_recording_file_thread_detached;
+pthread_t request_listener_count_thread_detached;
+pthread_t url_song_update_thread_detached;
+
+ATOM_NEW_INT(url_song_update_thread_running);
+ATOM_NEW_INT(request_listener_count_thread_running);
+ATOM_NEW_INT(g_listener_count);
+ATOM_NEW_INT(failed_get_listener_tries);
 
 int g_vu_meter_timer_is_active = 0;
 int g_stop_vu_meter_timer = 0;
 
-void set_threshold_input_from_command(command_t *command, Fl_Value_Input *input)
+void set_threshold_input_from_command(command_t *command, Fl_Value_Input *input, Fl_Check_Button *checkbox)
 {
     if (command->param != NULL) {
-        input->value(*(int *)command->param);
-        input->do_callback();
+        float val = *(float *)command->param;
+        if (val <= 0) {
+            checkbox->value(0);
+            checkbox->do_callback();
+        }
+        else {
+            checkbox->value(1);
+            checkbox->do_callback();
+            input->value(val);
+            input->do_callback();
+        }
         free(command->param);
     }
 }
@@ -148,19 +167,19 @@ void cmd_timer(void *)
         }
         break;
     case CMD_SET_STREAM_SIGNAL_THRESHOLD:
-        set_threshold_input_from_command(&command, fl_g->input_cfg_signal);
+        set_threshold_input_from_command(&command, fl_g->input_cfg_signal, fl_g->check_stream_signal);
         Fl::repeat_timeout(0.25, &cmd_timer);
         break;
     case CMD_SET_STREAM_SILENCE_THRESHOLD:
-        set_threshold_input_from_command(&command, fl_g->input_cfg_silence);
+        set_threshold_input_from_command(&command, fl_g->input_cfg_silence, fl_g->check_stream_silence);
         Fl::repeat_timeout(0.25, &cmd_timer);
         break;
     case CMD_SET_RECORD_SIGNAL_THRESHOLD:
-        set_threshold_input_from_command(&command, fl_g->input_rec_signal);
+        set_threshold_input_from_command(&command, fl_g->input_rec_signal, fl_g->check_rec_signal);
         Fl::repeat_timeout(0.25, &cmd_timer);
         break;
     case CMD_SET_RECORD_SILENCE_THRESHOLD:
-        set_threshold_input_from_command(&command, fl_g->input_rec_silence);
+        set_threshold_input_from_command(&command, fl_g->input_rec_silence, fl_g->check_rec_silence);
         Fl::repeat_timeout(0.25, &cmd_timer);
         break;
     case CMD_GET_STATUS:
@@ -176,7 +195,7 @@ void cmd_timer(void *)
         if (connected == 1) {
             status_packet.stream_seconds = (uint32_t)timer_get_elapsed_time(&stream_timer);
             status_packet.stream_kByte = kbytes_sent;
-            status_packet.listener_count = g_listener_count;
+            status_packet.listener_count = atom_get_int(&g_listener_count);
         }
         else {
             status_packet.stream_seconds = 0;
@@ -285,8 +304,7 @@ void vu_meter_timer(void *)
 void display_info_timer(void *)
 {
     char lcd_text_buf[33];
-    
-    
+
     if (try_to_connect == 1) {
         Fl::repeat_timeout(0.2, &display_info_timer);
         return;
@@ -302,6 +320,21 @@ void display_info_timer(void *)
         print_lcd(lcd_text_buf, strlen(lcd_text_buf), 0, 1);
     }
 
+    if (display_info == STREAM_STATE && connected == true) {
+        if (cfg.gui.show_listeners == true && atom_get_int(&g_listener_count) > -1) {
+            if (atom_get_int(&g_listener_count) <= 99999) {
+                snprintf(lcd_text_buf, sizeof(lcd_text_buf), _("On Air\nListeners %5d"), atom_get_int(&g_listener_count));
+            }
+            else {
+                snprintf(lcd_text_buf, sizeof(lcd_text_buf), _("On Air\nListeners>%5d"), atom_get_int(&g_listener_count));
+            }
+        }
+        else {
+            snprintf(lcd_text_buf, sizeof(lcd_text_buf), _("On Air"));
+        }
+        print_lcd(lcd_text_buf, strlen(lcd_text_buf), 0, 1);
+    }
+
     if (display_info == REC_TIME && timer_is_elapsed(&rec_timer)) {
         snprintf(lcd_text_buf, sizeof(lcd_text_buf), _("record time\n%s"), timer_get_time_str(&rec_timer));
         print_lcd(lcd_text_buf, strlen(lcd_text_buf), 0, 1);
@@ -312,9 +345,48 @@ void display_info_timer(void *)
         print_lcd(lcd_text_buf, strlen(lcd_text_buf), 0, 1);
     }
 
+    if (cfg.gui.show_listeners == true) {
+        if (display_info == STREAM_STATE) {
+            fl_g->label_current_listeners->hide();
+        }
+        else if (connected && cfg.srv[cfg.selected_srv]->type != WEBRTC && atom_get_int(&g_listener_count) > -1) {
+            fl_g->label_current_listeners->show();
+        }
+    }
+
     Fl::repeat_timeout(0.2, &display_info_timer);
 }
 
+void rotate_sponsor_logo_timer(void *)
+{
+    static int current_logo = rand() % 2;
+    static Fl_Image *sponsor_logo = NULL;
+
+    if (sponsor_logo != NULL) {
+        delete sponsor_logo;
+    }
+
+    if (current_logo == 0) {
+        sponsor_logo = new Fl_RGB_Image(radio_co_logo, 124, 61, 4, 0);
+        fl_g->sponsor_logo->callback([](Fl_Widget *w, void *u) {
+            fl_open_uri("https://radio.co/?utm_source=butt&utm_medium=app");
+        });
+        current_logo = 1;
+    }
+    else if (current_logo == 1) {
+        sponsor_logo = new Fl_RGB_Image(live365_logo, 124, 61, 4, 0);
+        fl_g->sponsor_logo->callback([](Fl_Widget *w, void *u) {
+            fl_open_uri("https://live365.com/broadcaster/radio-broadcasting?utm_source=butt&utm_medium=click&utm_campaign=onestopshop");
+        });
+        current_logo = 0;
+    }
+
+    fl_g->sponsor_logo->image(sponsor_logo);
+    fl_g->sponsor_logo->redraw();
+
+    Fl::repeat_timeout(10, &rotate_sponsor_logo_timer);
+}
+
 void display_rotate_timer(void *)
 {
     if (!connected && !recording) {
@@ -326,6 +398,7 @@ void display_rotate_timer(void *)
     }
 
     switch (display_info) {
+    case STREAM_STATE:
     case STREAM_TIME:
         display_info = SENT_DATA;
         break;
@@ -356,13 +429,25 @@ exit:
     Fl::repeat_timeout(5, &display_rotate_timer);
 }
 
+void reconnect_timer(void *)
+{
+    if (disconnect == 1) {
+        return;
+    }
+    button_connect_cb();
+}
+
 void is_connected_timer(void *)
 {
     if (!connected) {
-        print_info(_("ERROR: Connection lost\nreconnecting..."), 1);
         if (cfg.srv[cfg.selected_srv]->type == ICECAST) {
             ic_disconnect();
         }
+#ifdef HAVE_LIBDATACHANNEL
+        else if (cfg.srv[cfg.selected_srv]->type == WEBRTC) {
+            webrtc_disconnect();
+        }
+#endif
         else {
             sc_disconnect();
         }
@@ -376,8 +461,24 @@ void is_connected_timer(void *)
         fl_g->button_connect->color(FL_BACKGROUND_COLOR);
         fl_g->button_connect->redraw();
 
-        // reconnect
-        button_connect_cb();
+        if (disconnect == 1) {
+            return;
+        }
+
+        // Initiate delayed reconnect
+        if (cfg.main.reconnect_delay > 1) {
+            char text_buf[256];
+            fl_g->lcd->clear();
+            fl_g->lcd->print((const uchar *)_("idle"), strlen(_("idle")));
+            fl_g->sponsor_logo->show();
+            fl_g->sponsor_logo->redraw();
+            snprintf(text_buf, sizeof(text_buf), _("ERROR: Connection lost\nreconnecting in %d seconds..."), cfg.main.reconnect_delay);
+            print_info(text_buf, 0);
+        }
+        else {
+            print_info(_("ERROR: Connection lost\nreconnecting..."), 1);
+        }
+        Fl::add_timeout(cfg.main.reconnect_delay, &reconnect_timer);
 
         return;
     }
@@ -432,71 +533,95 @@ void *split_recording_file_thread_func(void *init)
         nanosleep(&wait_400ms, NULL);
     }
 
+    // Detach thread (free ressources) because no one will call pthread_join() on it
+    pthread_detach(pthread_self());
+
     return NULL;
 }
 
-int request_listener_count_thread_running = 0;
-void *request_listener_count_thread_func(void *listener_count)
+void *request_listener_count_thread_func(void *reset)
 {
-    request_listener_count_thread_running = 1;
-    int *listeners = (int *)listener_count;
+    int listeners;
+    static int got_listener_count = 0;
+
+    atom_set_int(&request_listener_count_thread_running, 1);
+
     if (cfg.srv[cfg.selected_srv]->type == ICECAST) {
-        *listeners = ic_get_listener_count();
+        listeners = ic_get_listener_count();
     }
     else if (cfg.srv[cfg.selected_srv]->type == SHOUTCAST) {
-        *listeners = sc_get_listener_count();
+        listeners = sc_get_listener_count();
     }
-    else if (cfg.srv[cfg.selected_srv]->type == RADIOCO) {
-        *listeners = -1;
+#ifdef HAVE_LIBDATACHANNEL
+    else if (cfg.srv[cfg.selected_srv]->type == WEBRTC) {
+        listeners = webrtc_get_listener_count();
     }
-    request_listener_count_thread_running = 0;
+#endif
+    else { // if (cfg.srv[cfg.selected_srv]->type == RADIOCO)
+        listeners = -1;
+    }
+
+    // Increase failed_tries only if we never got a listener count for this session
+    if (listeners == -1 && got_listener_count == 0) {
+        int failed_tries = atom_get_int(&failed_get_listener_tries);
+        atom_set_int(&failed_get_listener_tries, ++failed_tries);
+        goto return_thread;
+    }
+
+    Fl::lock();
+    if (listeners >= 0) {
+        char listener_label[32];
+        if (display_info != STREAM_STATE) {
+            snprintf(listener_label, sizeof(listener_label), "%s: %d", _("Listeners"), listeners);
+            fl_g->label_current_listeners->copy_label(listener_label);
+            fl_g->label_current_listeners->show();
+        }
+        got_listener_count = 1;
+        atom_set_int(&g_listener_count, listeners);
+    }
+    else {
+        fl_g->label_current_listeners->hide();
+        atom_set_int(&g_listener_count, -1);
+    }
+    Fl::unlock();
+
+return_thread:
+    // Detach thread (free ressources) because no one will call pthread_join() on it
+    pthread_detach(pthread_self());
+
+    atom_set_int(&request_listener_count_thread_running, 0);
 
     return NULL;
 }
 
 void request_listener_count_timer(void *reset)
 {
-    char listener_label[32];
-    static int failed_tries = 0;
-    static int listener_count = -2;
-
     if (reset != NULL && *(int *)reset == 1) {
-        failed_tries = 0;
-        listener_count = -2;
-        g_listener_count = -1;
+        atom_set_int(&failed_get_listener_tries, 0);
+        atom_set_int(&g_listener_count, -1);
     }
 
-    if (connected == 0 || failed_tries > 3) {
+    if (connected == 0 || atom_get_int(&failed_get_listener_tries) > 3) {
         fl_g->label_current_listeners->hide();
         return;
     }
 
-    if (listener_count == -1) {
-        failed_tries++;
+    if (atom_get_int(&request_listener_count_thread_running) == 0) {
+        if (pthread_create(&request_listener_count_thread_detached, NULL, request_listener_count_thread_func, reset) != 0) {
+            print_info("Fatal error: Could not launch request listeners thread. Please restart BUTT", 1);
+            return;
+        }
     }
 
-    if (listener_count >= 0) {
-        fl_g->label_current_listeners->show();
-        snprintf(listener_label, sizeof(listener_label), "%s: %d", _("Listeners"), listener_count);
-        fl_g->label_current_listeners->copy_label(listener_label);
-        g_listener_count = listener_count;
-    }
-    else {
-        fl_g->label_current_listeners->hide();
-        g_listener_count = -1;
-    }
-
-    if (request_listener_count_thread_running == 0) {
-        listener_count = -2;
-        pthread_create(&request_listener_count_thread, NULL, request_listener_count_thread_func, &listener_count);
-    }
-
-    Fl::repeat_timeout(3, &request_listener_count_timer);
+    Fl::repeat_timeout(cfg.gui.listeners_update_rate, &request_listener_count_timer);
 }
 
 void split_recording_file_timer(void)
 {
-    pthread_create(&split_recording_file_thread, NULL, split_recording_file_thread_func, NULL);
+    if (pthread_create(&split_recording_file_thread_detached, NULL, split_recording_file_thread_func, NULL) != 0) {
+        print_info("Fatal error: Could not launch split recording thread. Please restart BUTT", 1);
+        return;
+    }
 }
 
 void split_recording_file(void)
@@ -523,9 +648,7 @@ void split_recording_file(void)
         return;
     }
 
-    if ((next_fd = fl_fopen(cfg.rec.path, "rb")) != NULL) {
-        fclose(next_fd);
-
+    if (fl_access(cfg.rec.path, F_OK) == 0) { // File exists
         original_path = strdup(cfg.rec.path);
         snprintf(ext, sizeof(ext), "%s", p_ext);
         // increment the index until we find a filename that doesn't exist yet
@@ -533,17 +656,16 @@ void split_recording_file(void)
 
             snprintf(file_num_str, sizeof(file_num_str), "-%d", i);
 
-            // find beginn of the file extension
+            // find beginning of the file extension
             insert_pos = strrstr(cfg.rec.path, ext) - 1;
 
             // Put index between end of file name end beginning of extension
             strinsrt(&cfg.rec.path, file_num_str, insert_pos);
 
-            if ((next_fd = fl_fopen(cfg.rec.path, "rb")) == NULL) {
+            if (fl_access(cfg.rec.path, F_OK) < 0) { // Found non-existing file name
                 free(original_path);
-                break; // found valid file name
+                break;
             }
-            fclose(next_fd);
 
             if (i == 0xFFFFFFFF) { // 2^32-1
                 print_info(_("Could not find a valid filename for next file"
@@ -558,7 +680,7 @@ void split_recording_file(void)
         }
     }
 
-    if ((next_fd = fl_fopen(cfg.rec.path, "wb")) == NULL) {
+    if ((next_fd = fl_fopen(cfg.rec.path, "wb+")) == NULL) {
         fl_alert(_("Could not open:\n%s"), cfg.rec.path);
         return;
     }
@@ -569,8 +691,17 @@ void split_recording_file(void)
     next_file = 1;
 }
 
+void reset_stream_signal_detection_timer(void)
+{
+    if (cfg.main.signal_detection == 1 && cfg.main.signal_threshold > 0) {
+        Fl::remove_timeout(&stream_signal_timer);
+        Fl::add_timeout(1, &stream_signal_timer);
+    }
+}
+
 void stream_signal_timer(void *)
 {
+    // printf("stream_signal_timer\n");
     static timer_ms_t signal_timer;
 
     if (signal_detected == true) {
@@ -589,7 +720,15 @@ void stream_signal_timer(void *)
         timer_stop(&signal_timer);
     }
 
-    Fl::repeat_timeout(1, &stream_signal_timer);
+    Fl::repeat_timeout(0.1, &stream_signal_timer);
+}
+
+void reset_record_signal_detection_timer(void)
+{
+    if (cfg.rec.signal_detection == 1 && cfg.rec.signal_threshold > 0) {
+        Fl::remove_timeout(&record_signal_timer);
+        Fl::add_timeout(1, &record_signal_timer);
+    }
 }
 
 void record_signal_timer(void *)
@@ -614,7 +753,15 @@ void record_signal_timer(void *)
         timer_stop(&signal_timer);
     }
 
-    Fl::repeat_timeout(1, &record_signal_timer);
+    Fl::repeat_timeout(0.1, &record_signal_timer);
+}
+
+void reset_stream_silence_detection_timer(void)
+{
+    if (cfg.main.silence_detection == 1 && cfg.main.silence_threshold > 0) {
+        Fl::remove_timeout(&stream_silence_timer);
+        Fl::add_timeout(1, &stream_silence_timer);
+    }
 }
 
 void stream_silence_timer(void *)
@@ -641,7 +788,15 @@ void stream_silence_timer(void *)
         timer_stop(&silence_timer);
     }
 
-    Fl::repeat_timeout(1, &stream_silence_timer);
+    Fl::repeat_timeout(0.1, &stream_silence_timer);
+}
+
+void reset_record_silence_detection_timer(void)
+{
+    if (cfg.rec.silence_detection == 1 && cfg.rec.silence_threshold > 0) {
+        Fl::remove_timeout(&record_silence_timer);
+        Fl::add_timeout(1, &record_silence_timer);
+    }
 }
 
 void record_silence_timer(void *)
@@ -666,7 +821,7 @@ void record_silence_timer(void *)
         timer_stop(&silence_timer);
     }
 
-    Fl::repeat_timeout(1, &record_silence_timer);
+    Fl::repeat_timeout(0.1, &record_silence_timer);
 }
 
 void wait_for_radioco_timer(void *)
@@ -728,6 +883,7 @@ void songfile_timer(void *user_data)
     int num_of_newlines;
     char song[501];
     char msg[100];
+    char *fgets_ret;
     float repeat_time = 1;
 
     int reset;
@@ -786,10 +942,6 @@ void songfile_timer(void *user_data)
 
         fseek(cfg.main.song_fd, -100, SEEK_END);
         len = fread(song, sizeof(char), 100, cfg.main.song_fd);
-        if (len == 0) {
-            fclose(cfg.main.song_fd);
-            goto exit;
-        }
 
         // Count number of lines within the last 100 characters of the file
         // Some programs add a new line to the end of the file and some don't
@@ -825,13 +977,12 @@ void songfile_timer(void *user_data)
 
             last_line = strrchr(song, '\n') + 1;
         }
-        else // file has only one line
-        {
+        else { // file has only one line
             // Remove newlines at end of file
-            if (song[len - 2] == '\r') { // Windows
+            if (len > 1 && song[len - 2] == '\r') { // Windows
                 song[len - 2] = '\0';
             }
-            else if (song[len - 1] == '\n') { // OSX, Linux
+            else if (len > 0 && song[len - 1] == '\n') { // OSX, Linux
                 song[len - 1] = '\0';
             }
             else {
@@ -855,29 +1006,41 @@ void songfile_timer(void *user_data)
     }
     else {
         // read first line
-        if (fgets(song, 500, cfg.main.song_fd) != NULL) {
-            len = strlen(song);
+        fgets_ret = fgets(song, 500, cfg.main.song_fd);
 
-            // remove newline character
-            if (song[len - 2] == '\r') { // Windows
-                song[len - 2] = '\0';
-            }
-            else if (song[len - 1] == '\n') { // OSX, Linux
-                song[len - 1] = '\0';
-            }
-
-            // Remove UTF-8 BOM
-            if ((uint8_t)song[0] == 0xEF && (uint8_t)song[1] == 0xBB && (uint8_t)song[2] == 0xBF) {
-                cfg.main.song = (char *)realloc(cfg.main.song, strlen(song) + 1);
-                strcpy(cfg.main.song, song + 3);
-            }
-            else {
-                cfg.main.song = (char *)realloc(cfg.main.song, strlen(song) + 1);
-                strcpy(cfg.main.song, song);
-            }
-
-            Fl::add_timeout(cfg.main.song_delay, &update_song);
+        if (fgets_ret == NULL && feof(cfg.main.song_fd) != 0) { // file is empty (0 bytes)
+            len = 0;
         }
+        else if (fgets_ret != NULL) { // Normal case
+            len = strlen(song);
+        }
+        else { // Some error occurred
+            fclose(cfg.main.song_fd);
+            goto exit;
+        }
+
+        // remove newline character
+        if (len > 1 && song[len - 2] == '\r') { // Windows
+            song[len - 2] = '\0';
+        }
+        else if (len > 0 && song[len - 1] == '\n') { // OSX, Linux
+            song[len - 1] = '\0';
+        }
+        else {
+            song[len] = '\0';
+        }
+
+        // Remove UTF-8 BOM
+        if ((uint8_t)song[0] == 0xEF && (uint8_t)song[1] == 0xBB && (uint8_t)song[2] == 0xBF) {
+            cfg.main.song = (char *)realloc(cfg.main.song, strlen(song) + 1);
+            strcpy(cfg.main.song, song + 3);
+        }
+        else {
+            cfg.main.song = (char *)realloc(cfg.main.song, strlen(song) + 1);
+            strcpy(cfg.main.song, song);
+        }
+
+        Fl::add_timeout(cfg.main.song_delay, &update_song);
     }
 
     fclose(cfg.main.song_fd);
@@ -886,6 +1049,63 @@ exit:
     Fl::repeat_timeout(repeat_time, &songfile_timer);
 }
 
+void *url_song_update_thread_func(void *user_data)
+{
+    char song[1024];
+    char *new_line;
+
+    atom_set_int(&url_song_update_thread_running, 1);
+
+    uint32_t ret = url_get(cfg.main.song_update_url, NULL, song, sizeof(song));
+    if (ret <= 0) {
+        goto return_thread;
+    }
+
+    // Remove new line characters
+    new_line = strchr(song, '\r');
+    if (new_line != NULL) {
+        *new_line = '\0';
+    }
+    new_line = strchr(song, '\n');
+    if (new_line != NULL) {
+        *new_line = '\0';
+    }
+
+    if (cfg.main.song != NULL) {
+        if (strncmp(song, cfg.main.song, sizeof(song)) == 0) {
+            goto return_thread;
+        }
+    }
+
+    cfg.main.song = (char *)realloc(cfg.main.song, strlen(song) + 1);
+    strcpy(cfg.main.song, song);
+
+    // Call Fl::add_timeout from main thread
+    Fl::awake([](void *) -> void {
+        Fl::add_timeout(cfg.main.song_delay, &update_song);
+    });
+
+return_thread:
+    // Detach thread (free ressources) because no one will call pthread_join() on it
+    pthread_detach(pthread_self());
+
+    atom_set_int(&url_song_update_thread_running, 0);
+
+    return NULL;
+}
+
+void song_url_timer(void *user_data)
+{
+    if (atom_get_int(&url_song_update_thread_running) == 0) {
+        if (pthread_create(&url_song_update_thread_detached, NULL, url_song_update_thread_func, user_data) != 0) {
+            print_info("Fatal error: Could not launch url song update thread. Please restart BUTT", 1);
+            return;
+        }
+    }
+
+    Fl::repeat_timeout(cfg.main.song_update_url_interval, &song_url_timer);
+}
+
 void app_timer(void *user_data)
 {
     int reset;
