@@ -25,15 +25,6 @@ int g_aac_lib_available = 0;
 
 #ifdef HAVE_LIBFDK_AAC
 
-#if !defined(__APPLE__) && !defined(WIN32) // LINUX
-#define aacEncOpen_butt          aacEncOpen
-#define aacEncoder_SetParam_butt aacEncoder_SetParam
-#define aacEncoder_GetParam_butt aacEncoder_GetParam
-#define aacEncEncode_butt        aacEncEncode
-#define aacEncInfo_butt          aacEncInfo
-#define aacEncClose_butt         aacEncClose
-#endif
-
 int aac_enc_init(aac_enc *aac)
 {
     int rc;
@@ -86,7 +77,7 @@ int aac_enc_init(aac_enc *aac)
     aacEncoder_SetParam_butt(aac->handle, AACENC_CHANNELMODE, aac->channel);
     aacEncoder_SetParam_butt(aac->handle, AACENC_CHANNELORDER, 1);
     aacEncoder_SetParam_butt(aac->handle, AACENC_BITRATE, aac->bitrate * 1000);
-    aacEncoder_SetParam_butt(aac->handle, AACENC_TRANSMUX, 2); // taken from the example aac-enc.c
+    aacEncoder_SetParam_butt(aac->handle, AACENC_TRANSMUX, 2); // taken from the example aac-enc.c (ADTS output)
     aacEncoder_SetParam_butt(aac->handle, AACENC_AFTERBURNER, aac->afterburner);
 
     if (aac->bitrate_mode == 1) { // VBR
@@ -106,6 +97,7 @@ int aac_enc_init(aac_enc *aac)
     printf("AAC afterburner: %d\n", aac->afterburner);
     printf("AAC bitrate_mode: %d (vbr: %d)\n\n", aac->bitrate_mode, vbr_mode);
     */
+    // printf("AAC bitrate: %d\n", aac->bitrate);
 
     return 0;
 }
@@ -135,8 +127,9 @@ void aac_enc_close(aac_enc *aac)
         return;
     }
 
-    while (aac->state == AAC_BUSY)
+    while (aac->state == AAC_BUSY) {
         ;
+    }
 
     if (aac->handle != NULL) {
         aacEncClose_butt(&aac->handle);
@@ -154,7 +147,6 @@ int aac_enc_encode(aac_enc *aac, float *pcm_float, char *enc_buf, int frames, in
         return 0;
     }
 
-    int rc;
     AACENC_BufDesc in_buf = {0};
     AACENC_BufDesc out_buf = {0};
     AACENC_InArgs in_args = {0};
@@ -204,15 +196,80 @@ int aac_enc_encode(aac_enc *aac, float *pcm_float, char *enc_buf, int frames, in
         }
     }
 
-    if ((rc = aacEncEncode_butt(aac->handle, &in_buf, &out_buf, &in_args, &out_args)) != AACENC_OK) {
+    if (aacEncEncode_butt(aac->handle, &in_buf, &out_buf, &in_args, &out_args) != AACENC_OK) {
         return 0;
     }
 
     aac->state = AAC_READY;
-
     return out_args.numOutBytes;
 }
 
+int aac_enc_flush_file(aac_enc *aac, FILE *file)
+{
+    AACENC_BufDesc in_buf = {0};
+    AACENC_BufDesc out_buf = {0};
+    AACENC_InArgs in_args = {0};
+    AACENC_OutArgs out_args = {0};
+
+    int ret;
+    int in_identifier = IN_AUDIO_DATA;
+    int in_size, in_elem_size;
+    int out_identifier = OUT_BITSTREAM_DATA;
+    int out_size, out_elem_size;
+    void *out_ptr;
+    char enc_buf[8192];
+    int32_t dummy_buf[1] = {0};
+
+    if (aac->handle == NULL) {
+        printf("handle == NULL");
+        return 1;
+    }
+
+    in_size = 0;
+    in_elem_size = sizeof(int32_t);
+
+    in_args.numInSamples = -1;
+
+    in_buf.numBufs = 1;
+    in_buf.bufs = (void **)&dummy_buf;
+    in_buf.bufferIdentifiers = &in_identifier;
+    in_buf.bufSizes = &in_size;
+    in_buf.bufElSizes = &in_elem_size;
+
+    out_ptr = enc_buf;
+    out_size = sizeof(enc_buf);
+    out_elem_size = 1;
+
+    out_buf.numBufs = 1;
+    out_buf.bufs = &out_ptr;
+    out_buf.bufferIdentifiers = &out_identifier;
+    out_buf.bufSizes = &out_size;
+    out_buf.bufElSizes = &out_elem_size;
+
+    aac->state = AAC_BUSY;
+
+    for (;;) {
+        printf("FLUSH LOOP\n");
+        if ((ret = aacEncEncode_butt(aac->handle, &in_buf, &out_buf, &in_args, &out_args)) != AACENC_OK) {
+            if (ret == AACENC_ENCODE_EOF) {
+                break;
+            }
+            else {
+                //NOTE: When using the asan profile on macOS the flushing fails
+                printf("AAC flushing error: 0x%04X\n", ret);
+                aac->state = AAC_READY;
+                return 1;
+            }
+        }
+
+        fwrite(enc_buf, sizeof(char), out_args.numOutBytes, file);
+    }
+
+    aac->state = AAC_READY;
+
+    return 0;
+}
+
 int aac_enc_get_samplerate(aac_enc *aac)
 {
     //    return aacEncoder_GetParam_butt(aac->handle, AACENC_SAMPLERATE);
