@@ -30,6 +30,7 @@
 #include <pthread.h>
 #if !defined(__APPLE__) && !defined(WIN32)
 #include <FL/Fl_File_Icon.H>
+#include <dlfcn.h>
 #endif
 
 #if defined(__APPLE__)
@@ -68,12 +69,13 @@
 #include "aac_encode.h"
 #endif
 
+#include "port_midi.h"
+
 bool recording;
 bool connected;
 bool streaming;
 bool disconnect;
-bool song_timeout_running;
-bool app_timeout_running;
+bool try_to_connect;
 
 int stream_socket;
 double kbytes_sent;
@@ -95,14 +97,12 @@ flac_enc flac_rec;
 #ifdef HAVE_LIBFDK_AAC
 aac_enc aac_stream;
 aac_enc aac_rec;
-#if defined(__APPLE__) || defined(WIN32)
 aacEncOpenPtr aacEncOpen_butt;
 aacEncoder_SetParamPtr aacEncoder_SetParam_butt;
 aacEncoder_GetParamPtr aacEncoder_GetParam_butt;
 aacEncEncodePtr aacEncEncode_butt;
 aacEncInfoPtr aacEncInfo_butt;
 aacEncClosePtr aacEncClose_butt;
-#endif
 #endif // ifdef HAVE_LIBFDK_AAC
 
 #endif // ifndef BUILD_CLIENT
@@ -111,7 +111,7 @@ void print_usage(void)
 {
 #ifndef BUILD_CLIENT
     printf(
-        "Usage: butt [-h | -v | -s [name] | -u <song name> | -M <streaming signal threshold> | -m <streaming silence threshold> | -O <recording signal threshold> | -o <recording silence threshold> | -Sdrtqn | -c <config_path>] [-A | -U | -x] [-a <addr>] [-p <port>]\n");
+        "Usage: butt [-h | -v | -s [name] | -u <song name> | -M <streaming signal threshold> | -m <streaming silence threshold> | -O <recording signal threshold> | -o <recording silence threshold> | -SLdrtqn | -c <config_path>] [-A | -U | -x] [-a <addr>] [-p <port>]\n");
 #else
     printf(
         "Usage: butt-client [-h | -v | -s [name] | -u <song name> | -M <streaming signal threshold> | -m <streaming silence threshold> | -O <recording signal threshold> | -o <recording silence threshold> | -USdrtqn ] [-a <addr>] [-p <port>]\n");
@@ -137,6 +137,7 @@ void load_AAC_lib(void)
 #endif
 
 #if defined(__APPLE__)
+#ifdef HAVE_LIBFDK_AAC
     void *dylib = dlopen("libfdk-aac.2.dylib", RTLD_LAZY);
 
     if (dylib == NULL) {
@@ -158,10 +159,28 @@ void load_AAC_lib(void)
 
     //  askForMicPermission();
 #endif
+#endif
 
 #if !defined(__APPLE__) && !defined(WIN32) // LINUX
 #ifdef HAVE_LIBFDK_AAC
-    g_aac_lib_available = 1;
+    void *dylib = dlopen("libfdk-aac.so", RTLD_LAZY);
+
+    if (dylib == NULL) { // Try other lib name
+        dylib = dlopen("libfdk-aac.so.2", RTLD_LAZY);
+    }
+
+    if (dylib != NULL) {
+        // typedef AACENC_ERROR(WINAPI *aacEncOpenPtr)(HANDLE_AACENCODER*, UINT, UINT);
+
+        aacEncOpen_butt = (aacEncOpenPtr)dlsym(dylib, "aacEncOpen");
+        aacEncoder_SetParam_butt = (aacEncoder_SetParamPtr)dlsym(dylib, "aacEncoder_SetParam");
+        aacEncoder_GetParam_butt = (aacEncoder_GetParamPtr)dlsym(dylib, "aacEncoder_GetParam");
+        aacEncEncode_butt = (aacEncEncodePtr)dlsym(dylib, "aacEncEncode");
+        aacEncInfo_butt = (aacEncInfoPtr)dlsym(dylib, "aacEncInfo");
+        aacEncClose_butt = (aacEncClosePtr)dlsym(dylib, "aacEncClose");
+
+        g_aac_lib_available = 1;
+    }
 #endif
 #endif
 }
@@ -265,7 +284,15 @@ void set_locale_from_system(void)
 #elif defined(WIN32)
     bindtextdomain(PACKAGE, "locale");
 #else // Linux
-    bindtextdomain(PACKAGE, LOCALEDIR);
+    char *p;
+    if ((p = fl_getenv("APPDIR")) != NULL) { // Set locale dir when launched from .AppImage
+        char path_to_locale_dir[PATH_MAX];
+        snprintf(path_to_locale_dir, sizeof(path_to_locale_dir), "%s/usr/share/locale", p);
+        bindtextdomain(PACKAGE, path_to_locale_dir);
+    }
+    else { // Set locale dir to dir found by ./configure
+        bindtextdomain(PACKAGE, LOCALEDIR);
+    }
 #endif
 
     textdomain(PACKAGE);
@@ -301,7 +328,15 @@ void set_locale_from_config(void)
 #elif defined(WIN32)
     bindtextdomain(PACKAGE, "locale");
 #else // Linux
-    bindtextdomain(PACKAGE, LOCALEDIR);
+    char *p;
+    if ((p = fl_getenv("APPDIR")) != NULL) { // Set locale dir when launched from .AppImage
+        char path_to_locale_dir[PATH_MAX];
+        snprintf(path_to_locale_dir, sizeof(path_to_locale_dir), "%s/usr/share/locale", p);
+        bindtextdomain(PACKAGE, path_to_locale_dir);
+    }
+    else { // Set locale dir to dir found by ./configure
+        bindtextdomain(PACKAGE, LOCALEDIR);
+    }
 #endif
 
     textdomain(PACKAGE);
@@ -311,7 +346,7 @@ void set_locale_from_config(void)
 
 int get_threshold_from_args(char opt, char *optarg, command_t *command)
 {
-    int threshold = atoi(optarg);
+    float threshold = atof(optarg);
     if (threshold < 0) {
         printf(_("Illegal argument: Threshold must be a non-negative number\n"));
         fflush(stdout);
@@ -334,10 +369,10 @@ int get_threshold_from_args(char opt, char *optarg, command_t *command)
             printf("Internal error: Illegal value for \"opt\"\n");
             return -1;
         }
-        command->param_size = sizeof(int);
-        command->param = (int *)malloc(command->param_size);
-        *((int *)command->param) = threshold;
-        printf(_("%c threshold set to %d\n"), opt, threshold);
+        command->param_size = sizeof(float);
+        command->param = (float *)malloc(command->param_size);
+        *((float *)command->param) = threshold;
+        printf(_("%c threshold set to %0.1f\n"), opt, threshold);
     }
     return threshold;
 }
@@ -355,8 +390,13 @@ int main(int argc, char *argv[])
     char server_addr[128];
     uint32_t song_len;
 
+    srand(time(NULL));
+
 #ifndef BUILD_CLIENT
+
     cfg_path = NULL;
+    // Activate support for multi threading
+    Fl::lock();
 #endif
 
     command_t command;
@@ -386,7 +426,7 @@ int main(int argc, char *argv[])
 #endif
 
     // Parse command line parameters
-    while ((opt = getopt(argc, argv, ":vhc:AUxs:drtnqu:a:p:SM:m:O:o:")) != -1) {
+    while ((opt = getopt(argc, argv, ":vhc:AULxs:drtnqu:a:p:SM:m:O:o:")) != -1) {
         switch (opt) {
 #ifndef BUILD_CLIENT
         case 'A':
@@ -399,6 +439,10 @@ int main(int argc, char *argv[])
             cfg_path = (char *)malloc((strlen(optarg) + 1) * sizeof(char));
             strcpy(cfg_path, optarg);
             break;
+        case 'L':
+            snd_print_devices();
+            return 0;
+            break;
 #endif
         case 'U':
             command_proto = SOCK_PROTO_UDP;
@@ -507,6 +551,7 @@ int main(int argc, char *argv[])
 #ifndef BUILD_CLIENT
             printf(_("\nOptions for operating mode:\n"
                      "-c\tPath to configuration file\n"
+                     "-L\tPrint available audio devices\n"
                      "-A\tCommand server will be accessible from your network/internet (default: localhost only)\n"
                      "-U\tCommand server will use UDP instead of TCP\n"
                      "-x\tDo not start a command server\n"
@@ -695,8 +740,10 @@ int main(int argc, char *argv[])
     SendMessage(fl_xid(fl_g->window_main), WM_SETICON, ICON_SMALL, (LPARAM)smallicon);
 #endif
 
+#ifndef BUILD_CLIENT
 #if defined(__APPLE__)
     askForMicPermission();
+#endif
 #endif
 
     lame_stream.gfp = NULL;
@@ -720,11 +767,11 @@ int main(int argc, char *argv[])
         cfg.audio.dev_num = 0;
         cfg.audio.dev_name = (char *)realloc(cfg.audio.dev_name, strlen(_("Default PCM device (default)")) + 1);
         strcpy(cfg.audio.dev_name, _("Default PCM device (default)"));
-        
+
         cfg.audio.dev2_num = -1;
         cfg.audio.dev2_name = (char *)realloc(cfg.audio.dev2_name, strlen(_("None")) + 1);
         strcpy(cfg.audio.dev2_name, _("None"));
-        
+
         if (snd_open_streams() != 0) {
             fl_alert(_("Could not open audio device.\nPlease select your preferred audio device in settings->audio"));
         }
@@ -736,18 +783,19 @@ int main(int argc, char *argv[])
 
     Fl::add_timeout(5, &display_rotate_timer);
     Fl::add_timeout(0.25, &cmd_timer);
+    Fl::add_timeout(0, &rotate_sponsor_logo_timer);
 
     if (cfg.main.connect_at_startup) {
         button_connect_cb();
     }
-    else if (cfg.main.signal_threshold > 0) {
+    else if (cfg.main.signal_detection == 1 && cfg.main.signal_threshold > 0) {
         Fl::add_timeout(1, &stream_signal_timer);
     }
 
     if (cfg.rec.rec_after_launch && !recording) {
         button_record_cb(false);
     }
-    else if (cfg.rec.signal_threshold > 0) {
+    else if (cfg.rec.signal_detection == 1 && cfg.rec.signal_threshold > 0) {
         Fl::add_timeout(1, &record_signal_timer);
     }
 
@@ -787,6 +835,19 @@ int main(int argc, char *argv[])
         }
     }
 
+    PmError midi_result = midi_init();
+    if (midi_result == 0) {
+        if (strcmp(cfg.midi.dev_name, "Disabled") != 0) {
+            if (midi_open_device(cfg.midi.dev_name) == 0) {
+                midi_start_polling();
+            }
+        }
+    }
+    else {
+        snprintf(info_buf, sizeof(info_buf), _("Could not initialize PortMidi: %s"), Pm_GetErrorText(midi_result));
+        print_info(info_buf, 1);
+    }
+
     fill_cfg_widgets();
 
 #ifdef WIN32
@@ -808,10 +869,35 @@ int main(int argc, char *argv[])
     fl_g->radio_add_srv_radioco->hide();
 #endif
 
+#ifndef HAVE_LIBDATACHANNEL
+    fl_g->radio_add_srv_webrtc->deactivate();
+    fl_g->radio_add_srv_webrtc->tooltip(_("butt was built without WebRTC support"));
+#endif
+
     if (cfg.gui.start_minimized == 1) {
         fl_g->window_main->iconize();
     }
 
+    // Make sure input field callbacks are called when the Enter key is pressed or when the field is released.
+    // Unfortunately this can not be configured in fluid, so we have to do it here.
+    fl_g->input_gui_vu_mid_range_start->when(FL_WHEN_ENTER_KEY | FL_WHEN_RELEASE);
+    fl_g->input_gui_vu_high_range_start->when(FL_WHEN_ENTER_KEY | FL_WHEN_RELEASE);
+    fl_g->input_cfg_silence->when(FL_WHEN_ENTER_KEY | FL_WHEN_RELEASE);
+    fl_g->input_cfg_signal->when(FL_WHEN_ENTER_KEY | FL_WHEN_RELEASE);
+    fl_g->input_rec_signal->when(FL_WHEN_ENTER_KEY | FL_WHEN_RELEASE);
+    fl_g->input_rec_silence->when(FL_WHEN_ENTER_KEY | FL_WHEN_RELEASE);
+    fl_g->input_gui_listeners_update_rate->when(FL_WHEN_ENTER_KEY | FL_WHEN_RELEASE);
+    fl_g->input_rec_filename->when(FL_WHEN_ENTER_KEY | FL_WHEN_RELEASE);
+    fl_g->input_stream_mp3_lowpass_freq->when(FL_WHEN_ENTER_KEY | FL_WHEN_RELEASE);
+    fl_g->input_stream_mp3_lowpass_width->when(FL_WHEN_ENTER_KEY | FL_WHEN_RELEASE);
+    fl_g->input_stream_mp3_highpass_freq->when(FL_WHEN_ENTER_KEY | FL_WHEN_RELEASE);
+    fl_g->input_stream_mp3_highpass_width->when(FL_WHEN_ENTER_KEY | FL_WHEN_RELEASE);
+    fl_g->input_rec_mp3_lowpass_freq->when(FL_WHEN_ENTER_KEY | FL_WHEN_RELEASE);
+    fl_g->input_rec_mp3_lowpass_width->when(FL_WHEN_ENTER_KEY | FL_WHEN_RELEASE);
+    fl_g->input_rec_mp3_highpass_freq->when(FL_WHEN_ENTER_KEY | FL_WHEN_RELEASE);
+    fl_g->input_rec_mp3_highpass_width->when(FL_WHEN_ENTER_KEY | FL_WHEN_RELEASE);
+
     return GUI_LOOP();
+
 #endif // BUILD_CLIENT
 }
