@@ -25,12 +25,11 @@
 #include "flac_encode.h"
 #include "aac_encode.h"
 
-extern bool recording;   // TRUE if butt is recording
-extern bool connected;   // TRUE if butt is connected to server
-extern bool disconnect;  // TRUE if butt should disconnect
+extern bool recording;  // TRUE if butt is recording
+extern bool connected;  // TRUE if butt is connected to server
+extern bool disconnect; // TRUE if butt should disconnect
 extern bool streaming;
-extern bool song_timeout_running; // TRUE if automatic song updating is running
-extern bool app_timeout_running;  // TRUE if automatic song updating from some app is running
+extern bool try_to_connect;
 
 extern int stream_socket;
 extern double kbytes_sent;
