@@ -26,6 +26,7 @@
 #include "util.h"
 #include "fl_funcs.h"
 #include "strfuncs.h"
+#include "port_midi.h"
 
 #ifdef WIN32
 const char CONFIG_FILE[] = "buttrc";
@@ -33,7 +34,7 @@ const char CONFIG_FILE[] = "buttrc";
 const char CONFIG_FILE[] = ".buttrc";
 #endif
 
-const char *lang_array_new[] = {"system", "de", "en", "es", "fr", "pt_BR"};
+const char *lang_array_new[] = {"system", "ar", "de", "en", "es", "fr", "nl", "pt_BR"};
 const char *lang_array_old[] = {"system", "de", "en", "fr", "pt_BR"};
 
 config_t cfg;
@@ -89,6 +90,16 @@ int cfg_write_file(char *path)
             "num_of_icy = %d\n",
             cfg.main.num_of_srv, cfg.main.num_of_icy);
 
+    fprintf(cfg_fd, "song_update_url_active = %d\n", cfg.main.song_update_url_active);
+    fprintf(cfg_fd, "song_update_url_interval = %d\n", cfg.main.song_update_url_interval);
+
+    if (cfg.main.song_update_url != NULL) {
+        fprintf(cfg_fd, "song_update_url = %s\n", cfg.main.song_update_url);
+    }
+    else {
+        fprintf(cfg_fd, "song_update_url =\n");
+    }
+
     if (cfg.main.song_path != NULL) {
         fprintf(cfg_fd, "song_path = %s\n", cfg.main.song_path);
     }
@@ -125,13 +136,16 @@ int cfg_write_file(char *path)
 
     fprintf(cfg_fd, "gain = %f\n", cfg.main.gain);
 
-    fprintf(cfg_fd, "silence_threshold = %d\n", cfg.main.silence_threshold);
-    fprintf(cfg_fd, "signal_threshold = %d\n", cfg.main.signal_threshold);
+    fprintf(cfg_fd, "signal_threshold = %f\n", cfg.main.signal_threshold);
+    fprintf(cfg_fd, "silence_threshold = %f\n", cfg.main.silence_threshold);
+    fprintf(cfg_fd, "signal_detection = %d\n", cfg.main.signal_detection);
+    fprintf(cfg_fd, "silence_detection = %d\n", cfg.main.silence_detection);
     fprintf(cfg_fd, "check_for_update = %d\n", cfg.main.check_for_update);
     fprintf(cfg_fd, "start_agent = %d\n", cfg.main.start_agent);
     fprintf(cfg_fd, "minimize_to_tray = %d\n", cfg.main.minimize_to_tray);
     fprintf(cfg_fd, "connect_at_startup = %d\n", cfg.main.connect_at_startup);
     fprintf(cfg_fd, "force_reconnecting = %d\n", cfg.main.force_reconnecting);
+    fprintf(cfg_fd, "reconnect_delay = %d\n", cfg.main.reconnect_delay);
 
     if (cfg.main.ic_charset != NULL) {
         fprintf(cfg_fd, "ic_charset = %s\n", cfg.main.ic_charset);
@@ -190,14 +204,18 @@ int cfg_write_file(char *path)
             "start_rec = %d\n"
             "stop_rec = %d\n"
             "rec_after_launch = %d\n"
+            "overwrite_files = %d\n"
             "sync_to_hour  = %d\n"
             "split_time = %d\n"
             "filename = %s\n"
-            "silence_threshold = %d\n"
-            "signal_threshold = %d\n"
+            "signal_threshold = %f\n"
+            "silence_threshold = %f\n"
+            "signal_detection = %d\n"
+            "silence_detection = %d\n"
             "folder = %s\n\n",
-            cfg.rec.bitrate, cfg.rec.codec, cfg.rec.start_rec, cfg.rec.stop_rec, cfg.rec.rec_after_launch, cfg.rec.sync_to_hour, cfg.rec.split_time,
-            cfg.rec.filename, cfg.rec.silence_threshold, cfg.rec.signal_threshold, cfg.rec.folder);
+            cfg.rec.bitrate, cfg.rec.codec, cfg.rec.start_rec, cfg.rec.stop_rec, cfg.rec.rec_after_launch, cfg.rec.overwrite_files, cfg.rec.sync_to_hour,
+            cfg.rec.split_time, cfg.rec.filename, cfg.rec.signal_threshold, cfg.rec.silence_threshold, cfg.rec.signal_detection, cfg.rec.silence_detection,
+            cfg.rec.folder);
 
     fprintf(cfg_fd,
             "[tls]\n"
@@ -208,6 +226,7 @@ int cfg_write_file(char *path)
     fprintf(cfg_fd,
             "[dsp]\n"
             "equalizer = %d\n"
+            "equalizer_rec = %d\n"
             "eq_preset = %s\n"
             "gain1 = %f\n"
             "gain2 = %f\n"
@@ -220,15 +239,16 @@ int cfg_write_file(char *path)
             "gain9 = %f\n"
             "gain10 = %f\n"
             "compressor = %d\n"
+            "compressor_rec = %d\n"
             "aggressive_mode = %d\n"
             "threshold = %f\n"
             "ratio = %f\n"
             "attack = %f\n"
             "release = %f\n"
             "makeup_gain = %f\n\n",
-            cfg.dsp.equalizer, cfg.dsp.eq_preset, cfg.dsp.eq_gain[0], cfg.dsp.eq_gain[1], cfg.dsp.eq_gain[2], cfg.dsp.eq_gain[3], cfg.dsp.eq_gain[4],
-            cfg.dsp.eq_gain[5], cfg.dsp.eq_gain[6], cfg.dsp.eq_gain[7], cfg.dsp.eq_gain[8], cfg.dsp.eq_gain[9], cfg.dsp.compressor, cfg.dsp.aggressive_mode,
-            cfg.dsp.threshold, cfg.dsp.ratio, cfg.dsp.attack, cfg.dsp.release, cfg.dsp.makeup_gain);
+            cfg.dsp.equalizer_stream, cfg.dsp.equalizer_rec, cfg.dsp.eq_preset, cfg.dsp.eq_gain[0], cfg.dsp.eq_gain[1], cfg.dsp.eq_gain[2], cfg.dsp.eq_gain[3],
+            cfg.dsp.eq_gain[4], cfg.dsp.eq_gain[5], cfg.dsp.eq_gain[6], cfg.dsp.eq_gain[7], cfg.dsp.eq_gain[8], cfg.dsp.eq_gain[9], cfg.dsp.compressor_stream,
+            cfg.dsp.compressor_rec, cfg.dsp.aggressive_mode, cfg.dsp.threshold, cfg.dsp.ratio, cfg.dsp.attack, cfg.dsp.release, cfg.dsp.makeup_gain);
 
     fprintf(cfg_fd,
             "[mixer]\n"
@@ -252,14 +272,24 @@ int cfg_write_file(char *path)
             "y_pos = %d\n"
             "window_height = %d\n"
             "lcd_auto = %d\n"
+            "default_stream_info = %d\n"
             "start_minimized = %d\n"
             "disable_gain_slider = %d\n"
             "show_listeners = %d\n"
+            "listeners_update_rate = %d\n"
             "lang_str = %s\n"
+            "vu_low_color = %d\n"
+            "vu_mid_color = %d\n"
+            "vu_high_color = %d\n"
+            "vu_mid_range_start = %d\n"
+            "vu_high_range_start = %d\n"
             "always_show_vu_tabs = %d\n"
+            "window_title = %s\n"
             "vu_mode = %d\n\n",
             cfg.gui.attach, cfg.gui.ontop, cfg.gui.hide_log_window, cfg.gui.remember_pos, cfg.gui.x_pos, cfg.gui.y_pos, cfg.gui.window_height, cfg.gui.lcd_auto,
-            cfg.gui.start_minimized, cfg.gui.disable_gain_slider, cfg.gui.show_listeners, cfg.gui.lang_str, cfg.gui.always_show_vu_tabs, cfg.gui.vu_mode);
+            cfg.gui.default_stream_info, cfg.gui.start_minimized, cfg.gui.disable_gain_slider, cfg.gui.show_listeners, cfg.gui.listeners_update_rate,
+            cfg.gui.lang_str, cfg.gui.vu_low_color, cfg.gui.vu_mid_color, cfg.gui.vu_high_color, cfg.gui.vu_mid_range_start, cfg.gui.vu_high_range_start,
+            cfg.gui.always_show_vu_tabs, cfg.gui.window_title, cfg.gui.vu_mode);
 
     fprintf(cfg_fd,
             "[mp3_codec_stream]\n"
@@ -268,9 +298,22 @@ int cfg_write_file(char *path)
             "bitrate_mode = %d\n"
             "vbr_quality = %d\n"
             "vbr_min_bitrate = %d\n"
-            "vbr_max_bitrate = %d\n\n",
+            "vbr_max_bitrate = %d\n"
+            "vbr_force_min_bitrate = %d\n"
+            "resampling_freq = %d\n"
+            "lowpass_freq_active = %d\n"
+            "lowpass_freq = %f\n"
+            "lowpass_width_active = %d\n"
+            "lowpass_width = %f\n"
+            "highpass_freq_active = %d\n"
+            "highpass_freq = %f\n"
+            "highpass_width_active = %d\n"
+            "highpass_width = %f\n\n",
             cfg.mp3_codec_stream.enc_quality, cfg.mp3_codec_stream.stereo_mode, cfg.mp3_codec_stream.bitrate_mode, cfg.mp3_codec_stream.vbr_quality,
-            cfg.mp3_codec_stream.vbr_min_bitrate, cfg.mp3_codec_stream.vbr_max_bitrate);
+            cfg.mp3_codec_stream.vbr_min_bitrate, cfg.mp3_codec_stream.vbr_max_bitrate, cfg.mp3_codec_stream.vbr_force_min_bitrate,
+            cfg.mp3_codec_stream.resampling_freq, cfg.mp3_codec_stream.lowpass_freq_active, cfg.mp3_codec_stream.lowpass_freq,
+            cfg.mp3_codec_stream.lowpass_width_active, cfg.mp3_codec_stream.lowpass_width, cfg.mp3_codec_stream.highpass_freq_active,
+            cfg.mp3_codec_stream.highpass_freq, cfg.mp3_codec_stream.highpass_width_active, cfg.mp3_codec_stream.highpass_width);
 
     fprintf(cfg_fd,
             "[mp3_codec_rec]\n"
@@ -279,9 +322,21 @@ int cfg_write_file(char *path)
             "bitrate_mode = %d\n"
             "vbr_quality = %d\n"
             "vbr_min_bitrate = %d\n"
-            "vbr_max_bitrate = %d\n\n",
+            "vbr_max_bitrate = %d\n"
+            "vbr_force_min_bitrate = %d\n"
+            "resampling_freq = %d\n"
+            "lowpass_freq_active = %d\n"
+            "lowpass_freq = %f\n"
+            "lowpass_width_active = %d\n"
+            "lowpass_width = %f\n"
+            "highpass_freq_active = %d\n"
+            "highpass_freq = %f\n"
+            "highpass_width_active = %d\n"
+            "highpass_width = %f\n\n",
             cfg.mp3_codec_rec.enc_quality, cfg.mp3_codec_rec.stereo_mode, cfg.mp3_codec_rec.bitrate_mode, cfg.mp3_codec_rec.vbr_quality,
-            cfg.mp3_codec_rec.vbr_min_bitrate, cfg.mp3_codec_rec.vbr_max_bitrate);
+            cfg.mp3_codec_rec.vbr_min_bitrate, cfg.mp3_codec_rec.vbr_max_bitrate, cfg.mp3_codec_rec.vbr_force_min_bitrate, cfg.mp3_codec_rec.resampling_freq,
+            cfg.mp3_codec_rec.lowpass_freq_active, cfg.mp3_codec_rec.lowpass_freq, cfg.mp3_codec_rec.lowpass_width_active, cfg.mp3_codec_rec.lowpass_width,
+            cfg.mp3_codec_rec.highpass_freq_active, cfg.mp3_codec_rec.highpass_freq, cfg.mp3_codec_rec.highpass_width_active, cfg.mp3_codec_rec.highpass_width);
 
     fprintf(cfg_fd,
             "[vorbis_codec_stream]\n"
@@ -345,15 +400,45 @@ int cfg_write_file(char *path)
             "bit_depth = %d\n\n",
             cfg.wav_codec_rec.bit_depth);
 
-    for (i = 0; i < cfg.main.num_of_srv; i++) {
+    fprintf(cfg_fd,
+            "[midi]\n"
+            "dev_name = %s\n\n",
+            cfg.midi.dev_name);
+
+    for (i = 0; i < MIDI_COMMANDS_COUNT; i++) {
         fprintf(cfg_fd,
-                "[%s]\n"
-                "address = %s\n"
-                "port = %u\n"
-                "password = %s\n"
-                "type = %d\n"
-                "tls = %d\n",
-                cfg.srv[i]->name, cfg.srv[i]->addr, cfg.srv[i]->port, cfg.srv[i]->pwd, cfg.srv[i]->type, cfg.srv[i]->tls);
+                "[midi_cmd_%u]\n"
+                "enabled = %u\n"
+                "channel = %u\n"
+                "msg_num = %u\n"
+                "msg_type = %u\n"
+                "mode = %u\n"
+                "soft_takeover = %u\n\n",
+                i, cfg.midi.commands[i].enabled, cfg.midi.commands[i].channel, cfg.midi.commands[i].msg_num, cfg.midi.commands[i].msg_type,
+                cfg.midi.commands[i].mode, cfg.midi.commands[i].soft_takeover);
+    }
+
+    for (i = 0; i < cfg.main.num_of_srv; i++) {
+        if (cfg.srv[i]->type != WEBRTC) {
+            fprintf(cfg_fd,
+                    "[%s]\n"
+                    "address = %s\n"
+                    "port = %u\n"
+                    "password = %s\n"
+                    "type = %d\n"
+                    "tls = %d\n",
+                    cfg.srv[i]->name, cfg.srv[i]->addr, cfg.srv[i]->port, cfg.srv[i]->pwd, cfg.srv[i]->type, cfg.srv[i]->tls);
+        }
+        else { // WEBRTC
+            fprintf(cfg_fd,
+                    "[%s]\n"
+                    "address = (none)\n"
+                    "port = 0\n"
+                    "password = (none)\n"
+                    "type = %d\n"
+                    "tls = 0\n",
+                    cfg.srv[i]->name, WEBRTC);
+        }
 
         if (cfg.srv[i]->cert_hash != NULL) {
             fprintf(cfg_fd, "cert_hash = %s\n", cfg.srv[i]->cert_hash);
@@ -364,19 +449,44 @@ int cfg_write_file(char *path)
 
         if (cfg.srv[i]->type == ICECAST) {
             fprintf(cfg_fd, "mount = %s\n", cfg.srv[i]->mount);
-            fprintf(cfg_fd, "usr = %s\n\n", cfg.srv[i]->usr);
+            fprintf(cfg_fd, "usr = %s\n", cfg.srv[i]->usr);
+            fprintf(cfg_fd, "protocol = %d\n", cfg.srv[i]->icecast_protocol);
         }
         else { // Shoutcast has no mountpoint and user
             fprintf(cfg_fd, "mount = (none)\n");
-            fprintf(cfg_fd, "usr = (none)\n\n");
+            fprintf(cfg_fd, "usr = (none)\n");
+        }
+
+        if (cfg.srv[i]->type == WEBRTC) {
+            if (cfg.srv[i]->webrtc_ice != NULL && strlen(cfg.srv[i]->webrtc_ice) > 0) {
+                fprintf(cfg_fd, "webrtc_ice = %s\n", cfg.srv[i]->webrtc_ice);
+            }
+            else {
+                fprintf(cfg_fd, "webrtc_ice = \n");
+            }
+
+            fprintf(cfg_fd, "webrtc_whip = %s\n", cfg.srv[i]->webrtc_whip);
+
+            if (cfg.srv[i]->webrtc_auth != NULL && strlen(cfg.srv[i]->webrtc_auth) > 0) {
+                fprintf(cfg_fd, "webrtc_auth = %s\n\n", cfg.srv[i]->webrtc_auth);
+            }
+            else {
+                fprintf(cfg_fd, "webrtc_auth = \n\n");
+            }
+        }
+        else {
+            fprintf(cfg_fd, "\n");
         }
     }
 
     for (i = 0; i < cfg.main.num_of_icy; i++) {
         fprintf(cfg_fd,
                 "[%s]\n"
+                "expand_variables = %d\n"
                 "pub = %s\n",
-                cfg.icy[i]->name, cfg.icy[i]->pub);
+                cfg.icy[i]->name,             //
+                cfg.icy[i]->expand_variables, //
+                cfg.icy[i]->pub);
 
         if (cfg.icy[i]->desc != NULL) {
             fprintf(cfg_fd, "description = %s\n", cfg.icy[i]->desc);
@@ -480,11 +590,22 @@ int cfg_set_values(char *path)
     cfg.rec.start_rec = cfg_get_int("record", "start_rec", 0);
     cfg.rec.stop_rec = cfg_get_int("record", "stop_rec", 0);
     cfg.rec.rec_after_launch = cfg_get_int("record", "rec_after_launch", 0);
+    cfg.rec.overwrite_files = cfg_get_int("record", "overwrite_files", 0);
     cfg.rec.sync_to_hour = cfg_get_int("record", "sync_to_hour", 0);
     cfg.rec.split_time = cfg_get_int("record", "split_time", 0);
     cfg.rec.filename = cfg_get_str("record", "filename", "rec_%Y%m%d-%H%M%S_%i.mp3");
-    cfg.rec.silence_threshold = cfg_get_int("record", "silence_threshold", 0);
-    cfg.rec.signal_threshold = cfg_get_int("record", "signal_threshold", 0);
+    cfg.rec.signal_threshold = cfg_get_float("record", "signal_threshold", 0);
+    cfg.rec.silence_threshold = cfg_get_float("record", "silence_threshold", 0);
+    cfg.rec.signal_detection = cfg_get_int("record", "signal_detection", -1);
+    cfg.rec.silence_detection = cfg_get_int("record", "silence_detection", -1);
+
+    // Backwards compatibility with versions < 0.1.41
+    if (cfg.rec.signal_detection == -1) {
+        cfg.rec.signal_detection = cfg.rec.signal_threshold > 0 ? 1 : 0;
+    }
+    if (cfg.rec.silence_detection == -1) {
+        cfg.rec.silence_detection = cfg.rec.silence_threshold > 0 ? 1 : 0;
+    }
 
     cfg.rec.codec = cfg_get_str("record", "codec", "mp3");
     // Make sure that also "opus" and "flac" fit into the codec char array
@@ -531,12 +652,7 @@ int cfg_set_values(char *path)
 
     cfg.selected_srv = 0;
 
-    cfg.main.num_of_srv = cfg_get_int("main", "num_of_srv", -1);
-    if (cfg.main.num_of_srv == -1) {
-        fl_alert(_("error while parsing config. Illegal value (-1) for num_of_srv.\nbutt will start with default settings"));
-        return 1;
-    }
-
+    cfg.main.num_of_srv = cfg_get_int("main", "num_of_srv", 0);
     if (cfg.main.num_of_srv > 0) {
         cfg.main.srv = cfg_get_str("main", "server", NULL);
         if (cfg.main.srv == NULL) {
@@ -563,30 +679,6 @@ int cfg_set_values(char *path)
             cfg.srv[i]->name = (char *)malloc(strlen(srv_ent) + 1);
             snprintf(cfg.srv[i]->name, strlen(srv_ent) + 1, "%s", srv_ent);
 
-            cfg.srv[i]->addr = cfg_get_str(srv_ent, "address", NULL);
-            if (cfg.srv[i]->addr == NULL) {
-                fl_alert(_("error while parsing config. Missing address entry for server \"%s\"."
-                           "\nbutt will start with default settings"),
-                         srv_ent);
-                return 1;
-            }
-
-            cfg.srv[i]->port = cfg_get_int(srv_ent, "port", -1);
-            if (cfg.srv[i]->port == -1) {
-                fl_alert(_("error while parsing config. Missing port entry for server \"%s\"."
-                           "\nbutt will start with default settings"),
-                         srv_ent);
-                return 1;
-            }
-
-            cfg.srv[i]->pwd = cfg_get_str(srv_ent, "password", NULL);
-            if (cfg.srv[i]->pwd == NULL) {
-                fl_alert(_("error while parsing config. Missing password entry for server \"%s\"."
-                           "\nbutt will start with default settings"),
-                         srv_ent);
-                return 1;
-            }
-
             cfg.srv[i]->type = cfg_get_int(srv_ent, "type", -1);
             if (cfg.srv[i]->type == -1) {
                 fl_alert(_("error while parsing config. Missing type entry for server \"%s\"."
@@ -595,11 +687,39 @@ int cfg_set_values(char *path)
                 return 1;
             }
 
+            cfg.srv[i]->addr = cfg_get_str(srv_ent, "address", NULL);
+            if ((cfg.srv[i]->addr) == NULL && (cfg.srv[i]->type != WEBRTC)) {
+                fl_alert(_("error while parsing config. Missing address entry for server \"%s\"."
+                           "\nbutt will start with default settings"),
+                         srv_ent);
+                free(strtok_buf);
+                return 1;
+            }
+
+            cfg.srv[i]->port = cfg_get_int(srv_ent, "port", -1);
+            if ((cfg.srv[i]->port) == -1 && (cfg.srv[i]->type != WEBRTC)) {
+                fl_alert(_("error while parsing config. Missing port entry for server \"%s\"."
+                           "\nbutt will start with default settings"),
+                         srv_ent);
+                free(strtok_buf);
+                return 1;
+            }
+
+            cfg.srv[i]->pwd = cfg_get_str(srv_ent, "password", NULL);
+            if ((cfg.srv[i]->pwd == NULL) && (cfg.srv[i]->type != WEBRTC)) {
+                fl_alert(_("error while parsing config. Missing password entry for server \"%s\"."
+                           "\nbutt will start with default settings"),
+                         srv_ent);
+                free(strtok_buf);
+                return 1;
+            }
+
             cfg.srv[i]->mount = cfg_get_str(srv_ent, "mount", NULL);
             if ((cfg.srv[i]->mount == NULL) && (cfg.srv[i]->type == ICECAST)) {
                 fl_alert(_("error while parsing config. Missing mount entry for server \"%s\"."
                            "\nbutt will start with default settings"),
                          srv_ent);
+                free(strtok_buf);
                 return 1;
             }
 
@@ -608,6 +728,18 @@ int cfg_set_values(char *path)
                 cfg.srv[i]->usr = (char *)malloc(strlen("source") + 1);
                 strcpy(cfg.srv[i]->usr, "source");
             }
+
+            cfg.srv[i]->webrtc_ice = cfg_get_str(srv_ent, "webrtc_ice", NULL);
+
+            cfg.srv[i]->webrtc_whip = cfg_get_str(srv_ent, "webrtc_whip", NULL);
+            if ((cfg.srv[i]->webrtc_whip == NULL) && (cfg.srv[i]->type == WEBRTC)) {
+                cfg.srv[i]->webrtc_whip = (char *)malloc(strlen("(none)") + 1);
+                strcpy(cfg.srv[i]->webrtc_whip, "(none)");
+            }
+
+            cfg.srv[i]->webrtc_auth = cfg_get_str(srv_ent, "webrtc_auth", NULL);
+            cfg.srv[i]->icecast_protocol = cfg_get_int(srv_ent, "protocol", ICECAST_PROTOCOL_PUT);
+
 #ifdef HAVE_LIBSSL
             cfg.srv[i]->tls = cfg_get_int(srv_ent, "tls", 0);
 #else
@@ -631,11 +763,7 @@ int cfg_set_values(char *path)
         cfg.main.srv = NULL;
     }
 
-    cfg.main.num_of_icy = cfg_get_int("main", "num_of_icy", -1);
-    if (cfg.main.num_of_icy == -1) {
-        fl_alert(_("error while parsing config. Illegal value (-1) for num_of_icy.\nbutt will start with default settings"));
-        return 1;
-    }
+    cfg.main.num_of_icy = cfg_get_int("main", "num_of_icy", 0);
 
     cfg.selected_icy = 0;
     if (cfg.main.num_of_icy > 0) {
@@ -670,10 +798,12 @@ int cfg_set_values(char *path)
             cfg.icy[i]->icq = cfg_get_str(icy_ent, "icq", NULL);
             cfg.icy[i]->aim = cfg_get_str(icy_ent, "aim", NULL);
             cfg.icy[i]->pub = cfg_get_str(icy_ent, "pub", NULL);
+            cfg.icy[i]->expand_variables = cfg_get_int(icy_ent, "expand_variables", 0);
             if (cfg.icy[i]->pub == NULL) {
                 fl_alert(_("error while parsing config. Missing pub entry for icy \"%s\"."
                            "\nbutt will start with default settings"),
                          icy_ent);
+                free(strtok_buf);
                 return 1;
             }
 
@@ -701,6 +831,11 @@ int cfg_set_values(char *path)
         strrpl(&cfg.main.song_suffix, (char *)"%20", (char *)" ", MODE_ALL);
     }
 
+    // song update from URL
+    cfg.main.song_update_url_active = cfg_get_int("main", "song_update_url_active", 0);
+    cfg.main.song_update_url_interval = cfg_get_int("main", "song_update_url_interval", 1);
+    cfg.main.song_update_url = cfg_get_str("main", "song_update_url", NULL);
+
     // song update from file is default set to off
     cfg.main.song_update = cfg_get_int("main", "song_update", 0);
     cfg.main.song_delay = cfg_get_int("main", "song_delay", 0);
@@ -708,10 +843,22 @@ int cfg_set_values(char *path)
     cfg.main.app_update = cfg_get_int("main", "app_update", 0);
     cfg.main.app_update_service = cfg_get_int("main", "app_update_service", 0);
     cfg.main.app_artist_title_order = cfg_get_int("main", "app_artist_title_order", APP_TITLE_FIRST);
-    cfg.main.signal_threshold = cfg_get_int("main", "signal_threshold", 0);
-    cfg.main.silence_threshold = cfg_get_int("main", "silence_threshold", 0);
+    cfg.main.signal_threshold = cfg_get_float("main", "signal_threshold", 0);
+    cfg.main.silence_threshold = cfg_get_float("main", "silence_threshold", 0);
+    cfg.main.signal_detection = cfg_get_int("main", "signal_detection", -1);
+    cfg.main.silence_detection = cfg_get_int("main", "silence_detection", -1);
+
+    // Backwards compatibility with versions < 0.1.41
+    if (cfg.main.signal_detection == -1) {
+        cfg.main.signal_detection = cfg.main.signal_threshold > 0 ? 1 : 0;
+    }
+    if (cfg.main.silence_detection == -1) {
+        cfg.main.silence_detection = cfg.main.silence_threshold > 0 ? 1 : 0;
+    }
+
     cfg.main.connect_at_startup = cfg_get_int("main", "connect_at_startup", 0);
     cfg.main.force_reconnecting = cfg_get_int("main", "force_reconnecting", 0);
+    cfg.main.reconnect_delay = cfg_get_int("main", "reconnect_delay", 1);
     cfg.main.check_for_update = cfg_get_int("main", "check_for_update", 1);
     cfg.main.start_agent = cfg_get_int("main", "start_agent", 0);
     cfg.main.minimize_to_tray = cfg_get_int("main", "minimize_to_tray", 0);
@@ -734,7 +881,13 @@ int cfg_set_values(char *path)
     cfg.mixer.cross_fader = cfg_get_float("mixer", "cross_fader", 0.0);
 
     // DSP
-    cfg.dsp.equalizer = cfg_get_int("dsp", "equalizer", 0);
+    cfg.dsp.equalizer_stream = cfg_get_int("dsp", "equalizer", 0);
+    cfg.dsp.equalizer_rec = cfg_get_int("dsp", "equalizer_rec", -1);
+
+    // Backwards compatability with <= 0.1.40
+    if (cfg.dsp.equalizer_rec == -1) {
+        cfg.dsp.equalizer_rec = cfg.dsp.equalizer_stream;
+    }
 
     cfg.dsp.eq_preset = cfg_get_str("dsp", "eq_preset", "Manual");
 
@@ -749,13 +902,21 @@ int cfg_set_values(char *path)
     cfg.dsp.eq_gain[8] = cfg_get_float("dsp", "gain9", 0.0);
     cfg.dsp.eq_gain[9] = cfg_get_float("dsp", "gain10", NAN);
     if (isnan(cfg.dsp.eq_gain[9])) {
-        // Reset all EQ gains if band 10 has no config value. This prevents false values when the user updates from an earlier 5-band EQ to the new 10-band EQ
+        // Reset all EQ gains if band 10 has no config value. This prevents false values when the user updates from an earlier 5-band EQ to the new
+        // 10-band EQ
         for (int i = 0; i < EQ_BAND_COUNT; i++) {
             cfg.dsp.eq_gain[i] = 0.0;
         }
     }
 
-    cfg.dsp.compressor = cfg_get_int("dsp", "compressor", 0);
+    cfg.dsp.compressor_stream = cfg_get_int("dsp", "compressor", 0);
+    cfg.dsp.compressor_rec = cfg_get_int("dsp", "compressor_rec", -1);
+
+    // Backwards compatability with <= 0.1.40
+    if (cfg.dsp.compressor_rec == -1) {
+        cfg.dsp.compressor_rec = cfg.dsp.compressor_stream;
+    }
+
     cfg.dsp.aggressive_mode = cfg_get_int("dsp", "aggressive_mode", 0);
     cfg.dsp.threshold = cfg_get_float("dsp", "threshold", -20.0);
     cfg.dsp.ratio = cfg_get_float("dsp", "ratio", 5.0);
@@ -763,6 +924,19 @@ int cfg_set_values(char *path)
     cfg.dsp.release = cfg_get_float("dsp", "release", 1.0);
     cfg.dsp.makeup_gain = cfg_get_float("dsp", "makeup_gain", 0.0);
 
+    // MIDI
+    cfg.midi.dev_name = cfg_get_str("midi", "dev_name", "Disabled");
+    char midi_section[16];
+    for (i = 0; i < MIDI_COMMANDS_COUNT; i++) {
+        snprintf(midi_section, sizeof(midi_section), "midi_cmd_%d", i);
+        cfg.midi.commands[i].enabled = cfg_get_int(midi_section, "enabled", 0);
+        cfg.midi.commands[i].channel = cfg_get_int(midi_section, "channel", 0); // channel = 0 means all channels are allowed
+        cfg.midi.commands[i].msg_num = cfg_get_int(midi_section, "msg_num", i);
+        cfg.midi.commands[i].msg_type = cfg_get_int(midi_section, "msg_type", MIDI_MSG_TYPE_CC); // As of BUTT 0.1.41 "type" defaults to CC.
+        cfg.midi.commands[i].mode = cfg_get_int(midi_section, "mode", MIDI_MODE_ABSOLUTE);
+        cfg.midi.commands[i].soft_takeover = cfg_get_int(midi_section, "soft_takeover", 0);
+    }
+
     // read GUI stuff
     cfg.gui.attach = cfg_get_int("gui", "attach", 0);
     cfg.gui.ontop = cfg_get_int("gui", "ontop", 0);
@@ -772,12 +946,21 @@ int cfg_set_values(char *path)
     cfg.gui.y_pos = cfg_get_int("gui", "y_pos", -1);
     cfg.gui.window_height = cfg_get_int("gui", "window_height", 0);
     cfg.gui.lcd_auto = cfg_get_int("gui", "lcd_auto", 0);
+    cfg.gui.default_stream_info = cfg_get_int("gui", "default_stream_info", STREAM_TIME);
     cfg.gui.start_minimized = cfg_get_int("gui", "start_minimized", 0);
     cfg.gui.disable_gain_slider = cfg_get_int("gui", "disable_gain_slider", 0);
     cfg.gui.show_listeners = cfg_get_int("gui", "show_listeners", 1);
+    cfg.gui.listeners_update_rate = cfg_get_int("gui", "listeners_update_rate", 10);
     cfg.gui.vu_mode = cfg_get_int("gui", "vu_mode", VU_MODE_SOLID);
     cfg.gui.always_show_vu_tabs = cfg_get_int("gui", "always_show_vu_tabs", 1);
 
+    cfg.gui.vu_low_color = cfg_get_int("gui", "vu_low_color", (int)fl_rgb_color((uchar)0, (uchar)210, (uchar)0));   // Green
+    cfg.gui.vu_mid_color = cfg_get_int("gui", "vu_mid_color", (int)fl_rgb_color((uchar)230, (uchar)230, (uchar)0)); // Yellow
+    cfg.gui.vu_high_color = cfg_get_int("gui", "vu_high_color", (int)fl_rgb_color((uchar)200, (uchar)0, (uchar)0)); // Red
+
+    cfg.gui.vu_mid_range_start = cfg_get_int("gui", "vu_mid_range_start", -12); // dB
+    cfg.gui.vu_high_range_start = cfg_get_int("gui", "vu_high_range_start", -6);
+
     int lang_id = cfg_get_int("gui", "lang", -1);
     if (lang_id != -1) { // For compatibility with butt configurations <= 0.1.33
         cfg.gui.lang_str = NULL;
@@ -790,9 +973,15 @@ int cfg_set_values(char *path)
         }
     }
 
+    cfg.gui.window_title = cfg_get_str("gui", "window_title", NULL);
+    if (cfg.gui.window_title == NULL) {
+        cfg.gui.window_title = (char *)malloc(1);
+        cfg.gui.window_title[0] = '\0';
+    }
+
     // read FLTK related stuff
-    cfg.main.bg_color = cfg_get_int("main", "bg_color", 252645120); // dark gray
-    cfg.main.txt_color = cfg_get_int("main", "txt_color", -256);    // white
+    cfg.main.bg_color = cfg_get_int("main", "bg_color", (int)fl_rgb_color(15, 15, 15));      // dark gray
+    cfg.main.txt_color = cfg_get_int("main", "txt_color", (int)fl_rgb_color(255, 255, 255)); // white
 
     // read mp3 codec related stuff
     cfg.mp3_codec_stream.enc_quality = cfg_get_int("mp3_codec_stream", "enc_quality", 3);
@@ -801,6 +990,16 @@ int cfg_set_values(char *path)
     cfg.mp3_codec_stream.vbr_quality = cfg_get_int("mp3_codec_stream", "vbr_quality", 4);
     cfg.mp3_codec_stream.vbr_min_bitrate = cfg_get_int("mp3_codec_stream", "vbr_min_bitrate", 32);
     cfg.mp3_codec_stream.vbr_max_bitrate = cfg_get_int("mp3_codec_stream", "vbr_max_bitrate", 320);
+    cfg.mp3_codec_stream.vbr_force_min_bitrate = cfg_get_int("mp3_codec_stream", "vbr_force_min_bitrate", 0);
+    cfg.mp3_codec_stream.resampling_freq = cfg_get_int("mp3_codec_stream", "resampling_freq", CHOICE_MP3_RESAMPLING_DISABLED);
+    cfg.mp3_codec_stream.lowpass_freq_active = cfg_get_int("mp3_codec_stream", "lowpass_freq_active", 0);
+    cfg.mp3_codec_stream.lowpass_freq = cfg_get_float("mp3_codec_stream", "lowpass_freq", 0);
+    cfg.mp3_codec_stream.lowpass_width_active = cfg_get_int("mp3_codec_stream", "lowpass_width_active", 0);
+    cfg.mp3_codec_stream.lowpass_width = cfg_get_float("mp3_codec_stream", "lowpass_width", 0);
+    cfg.mp3_codec_stream.highpass_freq_active = cfg_get_int("mp3_codec_stream", "highpass_freq_active", 0);
+    cfg.mp3_codec_stream.highpass_freq = cfg_get_float("mp3_codec_stream", "highpass_freq", 0);
+    cfg.mp3_codec_stream.highpass_width_active = cfg_get_int("mp3_codec_stream", "highpass_width_active", 0);
+    cfg.mp3_codec_stream.highpass_width = cfg_get_float("mp3_codec_stream", "highpass_width", 0);
 
     cfg.mp3_codec_rec.enc_quality = cfg_get_int("mp3_codec_rec", "enc_quality", 3);
     cfg.mp3_codec_rec.stereo_mode = cfg_get_int("mp3_codec_rec", "stereo_mode", 0);
@@ -808,6 +1007,16 @@ int cfg_set_values(char *path)
     cfg.mp3_codec_rec.vbr_quality = cfg_get_int("mp3_codec_rec", "vbr_quality", 4);
     cfg.mp3_codec_rec.vbr_min_bitrate = cfg_get_int("mp3_codec_rec", "vbr_min_bitrate", 32);
     cfg.mp3_codec_rec.vbr_max_bitrate = cfg_get_int("mp3_codec_rec", "vbr_max_bitrate", 320);
+    cfg.mp3_codec_rec.vbr_force_min_bitrate = cfg_get_int("mp3_codec_rec", "vbr_force_min_bitrate", 0);
+    cfg.mp3_codec_rec.resampling_freq = cfg_get_int("mp3_codec_rec", "resampling_freq", CHOICE_MP3_RESAMPLING_DISABLED);
+    cfg.mp3_codec_rec.lowpass_freq_active = cfg_get_int("mp3_codec_rec", "lowpass_freq_active", 0);
+    cfg.mp3_codec_rec.lowpass_freq = cfg_get_float("mp3_codec_rec", "lowpass_freq", 0);
+    cfg.mp3_codec_rec.lowpass_width_active = cfg_get_int("mp3_codec_rec", "lowpass_width_active", 0);
+    cfg.mp3_codec_rec.lowpass_width = cfg_get_float("mp3_codec_rec", "lowpass_width", 0);
+    cfg.mp3_codec_rec.highpass_freq_active = cfg_get_int("mp3_codec_rec", "highpass_freq_active", 0);
+    cfg.mp3_codec_rec.highpass_freq = cfg_get_float("mp3_codec_rec", "highpass_freq", 0);
+    cfg.mp3_codec_rec.highpass_width_active = cfg_get_int("mp3_codec_rec", "highpass_width_active", 0);
+    cfg.mp3_codec_rec.highpass_width = cfg_get_float("mp3_codec_rec", "highpass_width", 0);
 
     // read ogg/vorbis codec related stuff
     cfg.vorbis_codec_stream.bitrate_mode = cfg_get_int("vorbis_codec_stream", "bitrate_mode", CHOICE_CBR);
@@ -919,6 +1128,9 @@ int cfg_create_default(void)
                     "num_of_icy = 0\n"
                     "srv_ent =\n"
                     "icy_ent =\n"
+                    "song_update_url_active = 0\n"
+                    "song_update_url_interval = 1\n"
+                    "song_update_url =\n"
                     "song_path =\n"
                     "song_update = 0\n"
                     "song_delay = 0\n"
@@ -931,12 +1143,15 @@ int cfg_create_default(void)
                     "log_file =\n"
                     "gain = 1.0\n"
                     "ic_charset =\n"
-                    "silence_threshhold = 0\n"
                     "signal_threshhold = 0\n"
-                    "check_for_update = 0\n"
+                    "silence_threshhold = 0\n"
+                    "signal_detection = 0\n"
+                    "silence_detection = 0\n"
+                    "check_for_update = 1\n"
                     "start_agent = 0\n"
                     "minimize_to_tray = 0\n"
                     "force_reconnecting = 0\n"
+                    "reconnect_delay = 1\n"
                     "connect_at_startup = 0\n\n");
 
     fprintf(cfg_fd,
@@ -961,11 +1176,14 @@ int cfg_create_default(void)
             "start_rec = 0\n"
             "stop_rec = 0\n"
             "rec_after_launch = 0\n"
+            "overwrite_files = 0\n"
             "sync_to_hour = 0\n"
             "split_time = 0\n"
             "filename = rec_%%Y%%m%%d-%%H%%M%%S.mp3\n"
-            "silence_threshhold = 0\n"
             "signal_threshhold = 0\n"
+            "silence_threshhold = 0\n"
+            "signal_detection = 0\n"
+            "silence_detection = 0\n"
             "folder = %s\n\n",
             def_rec_folder);
 
@@ -1000,9 +1218,12 @@ int cfg_create_default(void)
                     "hide_log_window = 0\n"
                     "remember_pos = 1\n"
                     "lcd_auto = 0\n"
+                    "default_stream_info = 0\n"
                     "start_minimized = 0\n"
                     "disable_gain_slider = 0\n"
                     "show_listeners = 1\n"
+                    "listeners_update_rate = 10\n"
+                    "window_title = \n"
                     "lang_str = system\n\n");
 
     fprintf(cfg_fd, "[mp3_codec_stream]\n"
@@ -1011,7 +1232,17 @@ int cfg_create_default(void)
                     "bitrate_mode = 0\n"
                     "vbr_quality = 4\n"
                     "vbr_min_bitrate = 32\n"
-                    "vbr_max_bitrate = 320\n\n");
+                    "vbr_max_bitrate = 320\n"
+                    "vbr_force_min_bitrate = 0\n"
+                    "resampling_freq = 0\n"
+                    "lowpass_freq_active = 0\n"
+                    "lowpass_freq = 0\n"
+                    "lowpass_width_active = 0\n"
+                    "lowpass_width = 0\n"
+                    "highpass_freq_active = 0\n"
+                    "highpass_freq = 0\n"
+                    "highpass_width_active = 0\n"
+                    "highpass_width = 0\n\n");
 
     fprintf(cfg_fd, "[mp3_codec_rec]\n"
                     "enc_quality = 3\n"
@@ -1019,7 +1250,17 @@ int cfg_create_default(void)
                     "bitrate_mode = 0\n"
                     "vbr_quality = 4\n"
                     "vbr_min_bitrate = 32\n"
-                    "vbr_max_bitrate = 320\n\n");
+                    "vbr_max_bitrate = 320\n"
+                    "vbr_force_min_bitrate = 0\n"
+                    "resampling_freq = 0\n"
+                    "lowpass_freq_active = 0\n"
+                    "lowpass_freq = 0\n"
+                    "lowpass_width_active = 0\n"
+                    "lowpass_width = 0\n"
+                    "highpass_freq_active = 0\n"
+                    "highpass_freq = 0\n"
+                    "highpass_width_active = 0\n"
+                    "highpass_width = 0\n\n");
 
     fprintf(cfg_fd, "[vorbis_codec_stream]\n"
                     "bitrate_mode = 0\n"
