@@ -25,6 +25,12 @@ enum {
     SHOUTCAST = 0,
     ICECAST = 1,
     RADIOCO = 2,
+    WEBRTC = 3,
+};
+
+enum {
+    ICECAST_PROTOCOL_PUT = 0,
+    ICECAST_PROTOCOL_SOURCE = 1,
 };
 
 enum {
@@ -60,12 +66,16 @@ enum {
 };
 
 enum {
-    LANG_SYSTEM = 0,
-    LANG_DE = 1,
-    LANG_EN = 2,
-    LANG_FR = 3,
-    LANG_PT_BR = 4,
-    LANG_ES = 5,
+    CHOICE_MP3_RESAMPLING_DISABLED = 0,
+    CHOICE_MP3_RESAMPLING_8000 = 1,
+    CHOICE_MP3_RESAMPLING_11025 = 2,
+    CHOICE_MP3_RESAMPLING_12000 = 3,
+    CHOICE_MP3_RESAMPLING_16000 = 4,
+    CHOICE_MP3_RESAMPLING_22050 = 5,
+    CHOICE_MP3_RESAMPLING_24000 = 6,
+    CHOICE_MP3_RESAMPLING_32000 = 7,
+    CHOICE_MP3_RESAMPLING_44100 = 8,
+    CHOICE_MP3_RESAMPLING_48000 = 9,
 };
 
 enum {
@@ -74,7 +84,11 @@ enum {
 };
 
 enum {
-    LANG_COUNT = 6,
+    LANG_COUNT = 8,
+};
+
+enum {
+    MIDI_COMMANDS_COUNT = 10,
 };
 
 enum {
@@ -100,14 +114,28 @@ typedef struct {
     char *name;
     char *addr;
     char *pwd;
-    char *mount;     // mountpoint for icecast server
-    char *usr;       // user for icecast server
-    char *cert_hash; // sha256 hash of trusted certificate
+    char *mount;       // mountpoint for icecast server
+    char *usr;         // user for icecast server
+    char *cert_hash;   // sha256 hash of trusted certificate
+    char *webrtc_ice;  // ICE server for WebRTC
+    char *webrtc_whip; // WHIP URL for WebRTC
+    char *webrtc_auth; // WHIP URL Bearer Token for WebRTC
     int port;
-    int type; // SHOUTCAST or ICECAST
-    int tls;  // use tls: 0 = no, 1 = yes
+    int type;             // SHOUTCAST, ICECAST or WEBRTC
+    int tls;              // use tls: 0 = no, 1 = yes
+    int icecast_protocol; // Icecast protocol: 0 = PUT, 1 = SOURCE
 } server_t;
 
+typedef struct {
+    int enabled;
+    int channel;       // Any, or 1-16
+    int msg_num;       // CC number
+    int msg_type;      // Currently unused
+    int mode;          // CC Mode. Absolute or relative
+    int soft_takeover; // If active, value jumps are prevented
+    int picked_up;     // 0 = not picked up, 1 = picked up (Only relevant if soft_takeover is active)
+} midi_command_t;
+
 typedef struct {
     char *name;
     char *desc; // description
@@ -117,6 +145,7 @@ typedef struct {
     char *icq;
     char *aim;
     char *pub;
+    int expand_variables;
 } icy_t;
 
 typedef struct {
@@ -139,13 +168,19 @@ typedef struct {
         int app_update;
         int app_update_service;
         int app_artist_title_order;
+        char *song_update_url;
+        int song_update_url_active;
+        int song_update_url_interval;
         int num_of_srv;
         int num_of_icy;
         int bg_color, txt_color;
         int connect_at_startup;
         int force_reconnecting;
-        int silence_threshold; // timeout duration of automatic stream/record stop
-        int signal_threshold;  // timeout duration of automatic stream/record start
+        int reconnect_delay;
+        float silence_threshold; // timeout duration of automatic stream stop
+        float signal_threshold;  // timeout duration of automatic stream start
+        int signal_detection;
+        int silence_detection;
         int check_for_update;
         int start_agent;
         int minimize_to_tray;
@@ -178,6 +213,11 @@ typedef struct {
         char *codec;
     } audio;
 
+    struct {
+        char *dev_name;
+        midi_command_t commands[MIDI_COMMANDS_COUNT];
+    } midi;
+
     struct {
         int channel;
         int bitrate;
@@ -190,11 +230,14 @@ typedef struct {
         FILE *fd;
         int start_rec;
         int stop_rec;
+        int overwrite_files;
         int rec_after_launch;
         int split_time;
         int sync_to_hour;
-        int silence_threshold;
-        int signal_threshold;
+        float silence_threshold;
+        float signal_threshold;
+        int signal_detection;
+        int silence_detection;
     } rec;
 
     struct {
@@ -203,10 +246,12 @@ typedef struct {
     } tls;
 
     struct {
-        int equalizer;
+        int equalizer_stream;
+        int equalizer_rec;
         char *eq_preset;
         double eq_gain[10];
-        int compressor;
+        int compressor_stream;
+        int compressor_rec;
         int aggressive_mode;
         double threshold, ratio, attack, release, makeup_gain;
     } dsp;
@@ -227,6 +272,7 @@ typedef struct {
         int attach;
         int ontop;
         int lcd_auto;
+        int default_stream_info;
         int hide_log_window;
         int remember_pos;
         int window_height;
@@ -234,9 +280,16 @@ typedef struct {
         char *lang_str;
         int vu_mode;
         int always_show_vu_tabs;
+        int vu_low_color;
+        int vu_mid_color;
+        int vu_high_color;
+        int vu_mid_range_start;
+        int vu_high_range_start;
         int start_minimized;
         int disable_gain_slider;
         int show_listeners;
+        int listeners_update_rate;
+        char *window_title;
     } gui;
 
     struct {
@@ -246,6 +299,17 @@ typedef struct {
         int vbr_quality;
         int vbr_min_bitrate;
         int vbr_max_bitrate;
+        int vbr_force_min_bitrate;
+        int resampling_freq;
+        int lowpass_freq_active;
+        float lowpass_freq;
+        int lowpass_width_active;
+        float lowpass_width;
+        int highpass_freq_active;
+        float highpass_freq;
+        int highpass_width_active;
+        float highpass_width;
+
     } mp3_codec_stream;
 
     struct {
@@ -255,6 +319,16 @@ typedef struct {
         int vbr_quality;
         int vbr_min_bitrate;
         int vbr_max_bitrate;
+        int vbr_force_min_bitrate;
+        int resampling_freq;
+        int lowpass_freq_active;
+        float lowpass_freq;
+        int lowpass_width_active;
+        float lowpass_width;
+        int highpass_freq_active;
+        float highpass_freq;
+        int highpass_width_active;
+        float highpass_width;
     } mp3_codec_rec;
 
     struct {
