@@ -232,6 +232,11 @@ const char *getCurrentTrackStrawberry(int artist_title_order)
     return get_mpris("org.mpris.MediaPlayer2.strawberry", artist_title_order);
 }
 
+const char *getCurrentTrackPlexamp(int artist_title_order)
+{
+    return get_mpris("org.mpris.MediaPlayer2.Plexamp", artist_title_order);
+}
+
 currentTrackFunction getCurrentTrackFunctionFromId(int i)
 {
     switch (i) {
@@ -247,6 +252,8 @@ currentTrackFunction getCurrentTrackFunctionFromId(int i)
         return &getCurrentTrackCantata;
     case 5:
         return &getCurrentTrackStrawberry;
+    case 6:
+        return &getCurrentTrackPlexamp;
     default:
         return NULL;
     }
