@@ -20,7 +20,7 @@
 #ifdef WIN32
 #include <winsock2.h>
 #define usleep(us) Sleep(us / 1000)
-//#define close(s) closesocket(s)
+// #define close(s) closesocket(s)
 #else
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -45,6 +45,7 @@
 #include "url.h"
 #include "flgui.h"
 #include "cJSON.h"
+#include "uri_encode.h"
 
 #ifdef HAVE_LIBSSL
 #include "tls.h"
@@ -69,12 +70,17 @@ int ic_connect(void)
     char recv_buf[1000];
     char send_buf[2048];
     char msg[256];
+    char *b64_enc;
     char *http_retval;
     static bool error_printed = 0;
 
     server_type = IC_TYPE_UNKNOWN;
+    memset(recv_buf, 0, sizeof(recv_buf));
 
     for (int try_cnt = 0; try_cnt < tries; try_cnt++) {
+        if (cfg.srv[cfg.selected_srv]->icecast_protocol == ICECAST_PROTOCOL_SOURCE) {
+            try_cnt++; // Start with SOURCE method and skip PUT try
+        }
         stream_socket = sock_connect(cfg.srv[cfg.selected_srv]->addr, cfg.srv[cfg.selected_srv]->port, SOCK_PROTO_TCP, CONN_TIMEOUT);
 
         if (stream_socket < 0) {
@@ -193,7 +199,9 @@ int ic_connect(void)
         }
 
         snprintf(auth, sizeof(auth), "%s:%s", cfg.srv[cfg.selected_srv]->usr, cfg.srv[cfg.selected_srv]->pwd);
-        snprintf(b64_auth, sizeof(b64_auth), "%s", util_base64_enc(auth));
+        b64_enc = util_base64_enc(auth);
+        snprintf(b64_auth, sizeof(b64_auth), "%s", b64_enc);
+        free(b64_enc);
         snprintf(send_buf + strlen(send_buf), sizeof(send_buf) - strlen(send_buf), "Authorization: Basic %s\r\n", b64_auth);
 
         // Make butt compatible to proxies/load balancers. Thanks to boyska
@@ -220,8 +228,19 @@ int ic_connect(void)
             snprintf(send_buf + strlen(send_buf), sizeof(send_buf) - strlen(send_buf), "Content-Type: audio/ogg\r\n");
         }
 
+        // Do not send bitrate information if flac is used
+        if (strcmp(cfg.audio.codec, "flac")) {
+            snprintf(send_buf + strlen(send_buf), sizeof(send_buf) - strlen(send_buf), "ice-bitrate: %d\r\n", cfg.audio.bitrate);
+        }
+
         if (cfg.main.num_of_icy > 0) {
-            snprintf(send_buf + strlen(send_buf), sizeof(send_buf) - strlen(send_buf), "ice-name: %s\r\n", cfg.icy[cfg.selected_icy]->name);
+            char *icy_name = strdup(cfg.icy[cfg.selected_icy]->name);
+            if (cfg.icy[cfg.selected_icy]->expand_variables == 1) {
+                expand_string(&icy_name);
+                strrpl(&icy_name, (char *)"%N", cfg.srv[cfg.selected_srv]->name, MODE_ALL);
+            }
+            snprintf(send_buf + strlen(send_buf), sizeof(send_buf) - strlen(send_buf), "ice-name: %s\r\n", icy_name);
+            free(icy_name);
         }
         else {
             snprintf(send_buf + strlen(send_buf), sizeof(send_buf) - strlen(send_buf), "ice-name: no name\r\n");
@@ -237,7 +256,16 @@ int ic_connect(void)
         if (cfg.main.num_of_icy > 0) {
             snprintf(send_buf + strlen(send_buf), sizeof(send_buf) - strlen(send_buf), "ice-url: %s\r\n", cfg.icy[cfg.selected_icy]->url);
             snprintf(send_buf + strlen(send_buf), sizeof(send_buf) - strlen(send_buf), "ice-genre: %s\r\n", cfg.icy[cfg.selected_icy]->genre);
-            snprintf(send_buf + strlen(send_buf), sizeof(send_buf) - strlen(send_buf), "ice-description: %s\r\n", cfg.icy[cfg.selected_icy]->desc);
+
+            if (cfg.icy[cfg.selected_icy]->desc != NULL && strlen(cfg.icy[cfg.selected_icy]->desc) > 0) {
+                char *icy_desc = strdup(cfg.icy[cfg.selected_icy]->desc);
+                if (cfg.icy[cfg.selected_icy]->expand_variables == 1) {
+                    expand_string(&icy_desc);
+                    strrpl(&icy_desc, (char *)"%N", cfg.srv[cfg.selected_srv]->name, MODE_ALL);
+                }
+                snprintf(send_buf + strlen(send_buf), sizeof(send_buf) - strlen(send_buf), "ice-description: %s\r\n", icy_desc);
+                free(icy_desc);
+            }
         }
 
         // Send audio settings
@@ -271,7 +299,7 @@ int ic_connect(void)
 
         ic_send(send_buf, (int)strlen(send_buf));
 
-        ret = ic_recv(recv_buf, sizeof(recv_buf));
+        ret = ic_recv(recv_buf, sizeof(recv_buf) - 1);
 
         // MARK: DEBUG
         // sprintf(recv_buf, "FOOHTTP/1.1 100 Continue\nContent-Length: 0\r\n\r\n");
@@ -290,12 +318,13 @@ int ic_connect(void)
         }
         if (ret == SOCK_TIMEOUT) {
             print_info(_("\nconnect: connection timed out. Trying again..."), 1);
-            usleep(100000);
+            usleep(100 * 1000);
             ic_disconnect();
             return IC_RETRY;
         }
         if (ret < 0) {
             // print_info("\nconnect: error while receiving server response\nThe server might require SSL/TLS", 1);
+            usleep(100 * 1000);
             ic_disconnect();
             return IC_RETRY;
         }
@@ -305,7 +334,7 @@ int ic_connect(void)
         // to see if the login was successfull (HTTP/1.0 200 OK)
         http_retval = strchr(temp, ' ');
         if (http_retval == NULL) {
-            usleep(100000);
+            usleep(100 * 1000);
             ic_disconnect();
             free(temp);
             return IC_RETRY;
@@ -465,6 +494,7 @@ int ic_update_song(char *song_name)
     char auth[150];
     char *song_buf;
     char *mount;
+    char *b64_enc;
 #ifdef HAVE_LIBSSL
     tls_t web_tls;
 #endif
@@ -504,15 +534,15 @@ int ic_update_song(char *song_name)
             web_tls.skip_verification = 1;
         }
 
-        if ((ret = tls_setup(&web_tls)) != TLS_OK) {
+        if (tls_setup(&web_tls) != TLS_OK) {
             sock_close(web_socket);
             return IC_ABORT;
         }
     }
 #endif
 
-    song_buf = strdup(song_name);
-    strencoderfc3986(&song_buf);
+    song_buf = (char *)malloc(strlen(song_name) * 3 + 1);
+    uri_encode(song_name, strlen(song_name), song_buf);
 
     mount = (char *)malloc(strlen(cfg.srv[cfg.selected_srv]->mount) + 2);
 
@@ -524,14 +554,14 @@ int ic_update_song(char *song_name)
     }
 
     snprintf(auth, sizeof(auth), "%s:%s", cfg.srv[cfg.selected_srv]->usr, cfg.srv[cfg.selected_srv]->pwd);
-
+    b64_enc = util_base64_enc(auth);
     if (cfg.main.ic_charset != NULL) {
         snprintf(send_buf, sizeof(send_buf),
                  "GET /admin/metadata?mode=updinfo&mount=%s&charset=%s&song=%s "
                  "HTTP/1.0\r\n"
                  "User-Agent: %s\r\n"
                  "Authorization: Basic %s\r\n",
-                 mount, cfg.main.ic_charset, song_buf, PACKAGE_STRING, util_base64_enc(auth));
+                 mount, cfg.main.ic_charset, song_buf, PACKAGE_STRING, b64_enc);
     }
     else {
         snprintf(send_buf, sizeof(send_buf),
@@ -539,8 +569,9 @@ int ic_update_song(char *song_name)
                  "HTTP/1.0\r\n"
                  "User-Agent: %s\r\n"
                  "Authorization: Basic %s\r\n",
-                 mount, song_buf, PACKAGE_STRING, util_base64_enc(auth));
+                 mount, song_buf, PACKAGE_STRING, b64_enc);
     }
+    free(b64_enc);
 
     if (cfg.srv[cfg.selected_srv]->port == 80) {
         snprintf(send_buf + strlen(send_buf), sizeof(send_buf) - strlen(send_buf), "Host: %s\r\n\r\n", cfg.srv[cfg.selected_srv]->addr);
@@ -552,7 +583,7 @@ int ic_update_song(char *song_name)
 
     if (cfg.srv[cfg.selected_srv]->tls == 1) {
 #ifdef HAVE_LIBSSL
-        ret = tls_send(&web_tls, send_buf, (int)strlen(send_buf), SEND_TIMEOUT);
+        tls_send(&web_tls, send_buf, (int)strlen(send_buf), SEND_TIMEOUT);
         tls_close(&web_tls);
 #endif
     }
