@@ -37,6 +37,7 @@
 #include "aac_encode.h"
 #include "shoutcast.h"
 #include "icecast.h"
+#include "webrtc.h"
 #include "strfuncs.h"
 #include "wav_header.h"
 #include "ringbuffer.h"
@@ -63,7 +64,6 @@ int framepacket_size;
 int framepacket_size2;
 int samplerate_dev2;
 
-bool try_to_connect;
 bool pa_new_frames;
 bool reconnect;
 bool silence_detected;
@@ -96,9 +96,9 @@ ATOM_NEW_COND(rec_cond);
 
 ATOM_NEW_INT(close_mixer_thread);
 
-pthread_t rec_thread;
-pthread_t stream_thread;
-pthread_t mixer_thread;
+pthread_t rec_thread_detached;
+pthread_t stream_thread_detached;
+pthread_t mixer_thread_joinable;
 
 PaStream *stream;
 PaStream *stream2;
@@ -119,7 +119,7 @@ void snd_reset_recording_compressor(void)
 void snd_set_vu_level_type(int type)
 {
     vu_level_type = type;
-    snd_update_vu(1);
+    vu_init(); // Reset peak indicators
 }
 
 int snd_init(void)
@@ -594,7 +594,11 @@ void snd_start_streaming_thread(void)
 
     kbytes_sent = 0;
     streaming = 1;
-    pthread_create(&stream_thread, NULL, snd_stream_thread, NULL);
+    if (pthread_create(&stream_thread_detached, NULL, snd_stream_thread, NULL) != 0) {
+        print_info("Fatal error: Could not launch streaming thread. Please restart BUTT", 1);
+        streaming = 0;
+        return;
+    }
 }
 
 void snd_start_mixer_thread(void)
@@ -609,13 +613,16 @@ void snd_start_mixer_thread(void)
     snd_reset_samplerate_conv(SND_STREAM);
     snd_reset_samplerate_conv(SND_REC);
 
-    pthread_create(&mixer_thread, NULL, snd_mixer_thread, NULL);
+    if (pthread_create(&mixer_thread_joinable, NULL, snd_mixer_thread, NULL) != 0) {
+        print_info("Fatal error: Could not launch mixer thread. Please restart BUTT", 1);
+        return;
+    }
 }
 
 void snd_stop_mixer_thread(void)
 {
     atom_set_int(&close_mixer_thread, 1);
-    pthread_join(mixer_thread, NULL);
+    pthread_join(mixer_thread_joinable, NULL);
 }
 
 void *snd_mixer_thread(void *data)
@@ -713,7 +720,11 @@ void *snd_mixer_thread(void *data)
                 stream_buf[i] *= cfg.mixer.streaming_gain;
             }
         }
+
+        streaming_dsp->eq_active = cfg.dsp.equalizer_stream;
+        streaming_dsp->drc_active = cfg.dsp.compressor_stream;
         if (streaming_dsp->hasToProcessSamples()) {
+            // printf("stream: \n");
             if (reset_streaming_compressor == true) {
                 streaming_dsp->reset_compressor();
                 reset_streaming_compressor = false;
@@ -738,7 +749,11 @@ void *snd_mixer_thread(void *data)
                 record_buf[i] *= cfg.mixer.recording_gain;
             }
         }
+
+        recording_dsp->eq_active = cfg.dsp.equalizer_rec;
+        recording_dsp->drc_active = cfg.dsp.compressor_rec;
         if (recording_dsp->hasToProcessSamples()) {
+            // printf("rec: \n");
             if (reset_recording_compressor == true) {
                 recording_dsp->reset_compressor();
                 reset_recording_compressor = false;
@@ -760,7 +775,7 @@ void *snd_mixer_thread(void *data)
         pa_new_frames = 1;
     }
 
-    return 0;
+    return NULL;
 }
 
 void *snd_stream_thread(void *data)
@@ -780,12 +795,16 @@ void *snd_stream_thread(void *data)
     if (cfg.srv[cfg.selected_srv]->type == ICECAST) {
         xc_send = &ic_send;
     }
-    else { // Icecast
+#ifdef HAVE_LIBDATACHANNEL
+    else if (cfg.srv[cfg.selected_srv]->type == WEBRTC) {
+        xc_send = &webrtc_send;
+    }
+#endif
+    else { // Shoutcast
         xc_send = &sc_send;
     }
 
     set_max_thread_priority();
-
     while (connected) {
         atom_cond_wait(&stream_cond);
         if (!connected) {
@@ -805,12 +824,17 @@ void *snd_stream_thread(void *data)
                 if (opus_stream.state == OPUS_STATE_NEW_STREAM) {
                     opus_enc_reinit(&opus_stream);
                     opus_enc_write_header(&opus_stream);
+                    encode_bytes_read = opus_enc_encode(&opus_stream, opus_stream.last_pcm_packet, enc_buf);
+                    if (xc_send(enc_buf, encode_bytes_read) == -1) {
+                        connected = 0;
+                        break;
+                    }
                 }
 
                 rb_read_len(&stream_rb, audio_buf, bytes_to_read);
-                encode_bytes_read = opus_enc_encode(&opus_stream, (float *)audio_buf, enc_buf, bytes_to_read / (cfg.audio.channel * sizeof(float)));
+                encode_bytes_read = opus_enc_encode(&opus_stream, (float *)audio_buf, enc_buf);
 
-                if ((sent = xc_send(enc_buf, encode_bytes_read)) == -1) {
+                if (xc_send(enc_buf, encode_bytes_read) == -1) {
                     connected = 0;
                 }
                 else {
@@ -826,7 +850,7 @@ void *snd_stream_thread(void *data)
                 encode_bytes_read =
                     aac_enc_encode(&aac_stream, (float *)audio_buf, enc_buf, bytes_to_read / (cfg.audio.channel * sizeof(float)), stream_rb.size * 10);
 
-                if ((sent = xc_send(enc_buf, encode_bytes_read)) == -1) {
+                if (xc_send(enc_buf, encode_bytes_read) == -1) {
                     connected = 0;
                 }
                 else {
@@ -868,7 +892,7 @@ void *snd_stream_thread(void *data)
                 }
             }
 
-            if ((sent = xc_send(enc_buf, encode_bytes_read)) == -1) {
+            if (xc_send(enc_buf, encode_bytes_read) == -1) {
                 connected = 0;
             }
             else {
@@ -880,6 +904,9 @@ void *snd_stream_thread(void *data)
     free(enc_buf);
     free(audio_buf);
 
+    // Detach thread (free ressources) because no one will call pthread_join() on it
+    pthread_detach(pthread_self());
+
     return NULL;
 }
 
@@ -902,7 +929,11 @@ void snd_start_recording_thread(void)
 
     atom_cond_init(&rec_cond);
 
-    pthread_create(&rec_thread, NULL, snd_rec_thread, NULL);
+    if (pthread_create(&rec_thread_detached, NULL, snd_rec_thread, NULL) != 0) {
+        print_info("Fatal error: Could not launch recording thread. Please restart BUTT", 1);
+        recording = 0;
+        return;
+    }
 
     print_info(_("Recording to:"), 0);
     print_info(cfg.rec.path, 0);
@@ -925,14 +956,13 @@ void *snd_rec_thread(void *data)
 {
     int rb_bytes_read;
     int bytes_to_read;
-    int ogg_header_written;
     int opus_header_written;
     int enc_bytes_read;
+    int buf_size = rec_rb.size * sizeof(char) * 10;
 
-    char *enc_buf = (char *)malloc(rec_rb.size * sizeof(char) * 10);
-    char *audio_buf = (char *)malloc(rec_rb.size * sizeof(char) * 10);
+    char *enc_buf = (char *)malloc(buf_size);
+    char *audio_buf = (char *)malloc(buf_size);
 
-    ogg_header_written = 0;
     opus_header_written = 0;
 
     set_max_thread_priority();
@@ -941,26 +971,51 @@ void *snd_rec_thread(void *data)
         atom_cond_wait(&rec_cond);
 
         if (next_file == 1) {
-            if (!strcmp(cfg.rec.codec, "flac")) { // The flac encoder closes the file
-                flac_enc_close(&flac_rec);
-            }
-            else {
-                fclose(cfg.rec.fd);
-            }
+#ifdef HAVE_LIBFDK_AAC
+            if (!strcmp(cfg.rec.codec, "aac")) {
+                aac_enc_flush_file(&aac_rec, cfg.rec.fd);
+                aac_enc_reinit(&aac_rec);
 
-            cfg.rec.fd = next_fd;
-            next_file = 0;
+                fclose(cfg.rec.fd);
+                cfg.rec.fd = next_fd;
+                next_file = 0;
+            }
+#endif
             if (!strcmp(cfg.rec.codec, "ogg")) {
+                // Flush encoder
+                enc_bytes_read = vorbis_enc_encode(&vorbis_rec, NULL, enc_buf, 0);
+                printf("flushed %d bytes\n", enc_bytes_read);
+                fwrite(enc_buf, 1, enc_bytes_read, cfg.rec.fd);
+                fclose(cfg.rec.fd);
+                cfg.rec.fd = next_fd;
+                next_file = 0;
+
+                // Re-init encoder for new file
                 vorbis_enc_reinit(&vorbis_rec);
-                ogg_header_written = 0;
             }
             if (!strcmp(cfg.rec.codec, "opus")) {
-                opus_enc_reinit(&opus_rec);
-                opus_header_written = 0;
+                opus_rec.state = OPUS_STATE_LAST_FRAME;
+                next_file = 0;
             }
             if (!strcmp(cfg.rec.codec, "flac")) {
+                flac_enc_close_file(&flac_rec);
                 flac_enc_reinit(&flac_rec);
-                flac_enc_init_FILE(&flac_rec, cfg.rec.fd);
+                flac_enc_init_FILE(&flac_rec, next_fd);
+                cfg.rec.fd = next_fd;
+                next_file = 0;
+            }
+            if (!strcmp(cfg.rec.codec, "mp3")) {
+                lame_enc_flush_file(&lame_rec, cfg.rec.fd);
+                lame_enc_reinit(&lame_rec);
+                fclose(cfg.rec.fd);
+                cfg.rec.fd = next_fd;
+                next_file = 0;
+            }
+            if (!strcmp(cfg.rec.codec, "wav")) {
+                wav_write_header(cfg.rec.fd, cfg.audio.channel, cfg.audio.samplerate, cfg.wav_codec_rec.bit_depth);
+                fclose(cfg.rec.fd);
+                cfg.rec.fd = next_fd;
+                next_file = 0;
             }
         }
 
@@ -978,8 +1033,18 @@ void *snd_rec_thread(void *data)
                     opus_header_written = 1;
                 }
 
-                enc_bytes_read = opus_enc_encode(&opus_rec, (float *)audio_buf, enc_buf, bytes_to_read / (cfg.audio.channel * sizeof(float)));
+                enc_bytes_read = opus_enc_encode(&opus_rec, (float *)audio_buf, enc_buf);
                 kbytes_written += fwrite(enc_buf, 1, enc_bytes_read, cfg.rec.fd) / 1024.0;
+
+                if (opus_rec.state == OPUS_STATE_NEW_STREAM) {
+                    fclose(cfg.rec.fd);
+                    cfg.rec.fd = next_fd;
+                    opus_enc_reinit(&opus_rec);
+                    opus_enc_write_header(&opus_rec);
+                    opus_header_written = 1;
+                    enc_bytes_read = opus_enc_encode(&opus_rec, opus_rec.last_pcm_packet, enc_buf);
+                    kbytes_written += fwrite(enc_buf, 1, enc_bytes_read, cfg.rec.fd) / 1024.0;
+                }
             }
         }
 #ifdef HAVE_LIBFDK_AAC
@@ -988,8 +1053,7 @@ void *snd_rec_thread(void *data)
             while ((rb_filled(&rec_rb)) >= bytes_to_read) {
                 rb_read_len(&rec_rb, audio_buf, bytes_to_read);
 
-                enc_bytes_read =
-                    aac_enc_encode(&aac_rec, (float *)audio_buf, enc_buf, bytes_to_read / (cfg.audio.channel * sizeof(float)), rec_rb.size * sizeof(char) * 10);
+                enc_bytes_read = aac_enc_encode(&aac_rec, (float *)audio_buf, enc_buf, bytes_to_read / (cfg.audio.channel * sizeof(float)), buf_size);
                 kbytes_written += fwrite(enc_buf, 1, enc_bytes_read, cfg.rec.fd) / 1024.0;
             }
         }
@@ -1005,14 +1069,13 @@ void *snd_rec_thread(void *data)
             }
 
             if (!strcmp(cfg.rec.codec, "mp3")) {
-                enc_bytes_read = lame_enc_encode(&lame_rec, (float *)audio_buf, enc_buf, rb_bytes_read / (cfg.audio.channel * sizeof(float)), rec_rb.size * 10);
+                enc_bytes_read = lame_enc_encode(&lame_rec, (float *)audio_buf, enc_buf, rb_bytes_read / (cfg.audio.channel * sizeof(float)), buf_size);
                 kbytes_written += fwrite(enc_buf, 1, enc_bytes_read, cfg.rec.fd) / 1024.0;
             }
 
             if (!strcmp(cfg.rec.codec, "ogg")) {
-                if (!ogg_header_written) {
+                if (vorbis_rec.header_written == 0) {
                     vorbis_enc_write_header(&vorbis_rec);
-                    ogg_header_written = 1;
                 }
 
                 enc_bytes_read = vorbis_enc_encode(&vorbis_rec, (float *)audio_buf, enc_buf, rb_bytes_read / (cfg.audio.channel * sizeof(float)));
@@ -1075,10 +1138,32 @@ void *snd_rec_thread(void *data)
     if (!strcmp(cfg.rec.codec, "wav")) {
         wav_write_header(cfg.rec.fd, cfg.audio.channel, cfg.audio.samplerate, cfg.wav_codec_rec.bit_depth);
     }
-
-    if (!strcmp(cfg.rec.codec, "flac")) { // The flac encoder closes the file
+    else if (!strcmp(cfg.rec.codec, "flac")) { // The flac encoder closes the file
         flac_enc_close_file(&flac_rec);
     }
+    else if (!strcmp(cfg.rec.codec, "mp3")) {
+        lame_enc_flush_file(&lame_rec, cfg.rec.fd);
+        lame_enc_reinit(&lame_rec); // Prepare for next recording
+        fclose(cfg.rec.fd);
+    }
+    else if (!strcmp(cfg.rec.codec, "ogg")) {
+        enc_bytes_read = vorbis_enc_encode(&vorbis_rec, NULL, enc_buf, 0);
+        vorbis_enc_reinit(&vorbis_rec);
+        fwrite(enc_buf, 1, enc_bytes_read, cfg.rec.fd);
+        fclose(cfg.rec.fd);
+    }
+
+#ifdef HAVE_LIBFDK_AAC
+    else if (!strcmp(cfg.rec.codec, "aac")) {
+        aac_enc_flush_file(&aac_rec, cfg.rec.fd);
+        aac_enc_reinit(&aac_rec);
+        fclose(cfg.rec.fd);
+    }
+#endif
+    else if (!strcmp(cfg.rec.codec, "opus")) {
+        opus_enc_reinit(&opus_rec);
+        fclose(cfg.rec.fd);
+    }
     else {
         fclose(cfg.rec.fd);
     }
@@ -1086,19 +1171,25 @@ void *snd_rec_thread(void *data)
     free(enc_buf);
     free(audio_buf);
 
+    // Detach thread (free ressources) because no one will call pthread_join() on it
+    pthread_detach(pthread_self());
+
     return NULL;
 }
 
 void snd_update_vu(int reset)
 {
     int i;
-    static float lpeak = 0;
-    static float rpeak = 0;
-    static int call_cnt = 1;
-    float *p;
     float decay = cfg.audio.samplerate < 88200 ? 0.5 : 0.7;
-    static double lavg = 0;
-    static double ravg = 0;
+    static float stream_lpeak = 0;
+    static float stream_rpeak = 0;
+    static float rec_lpeak = 0;
+    static float rec_rpeak = 0;
+    static int call_cnt = 1;
+    static double stream_lavg = 0;
+    static double stream_ravg = 0;
+    static double rec_lavg = 0;
+    static double rec_ravg = 0;
 
     if (reset == 1) {
         memset(stream_buf, 0, framepacket_size * sizeof(float) * cfg.audio.channel);
@@ -1106,86 +1197,98 @@ void snd_update_vu(int reset)
 
         vu_init(); // Reset peak indicators
         call_cnt = 1;
-        lpeak = 0;
-        rpeak = 0;
-        lavg = 0;
-        ravg = 0;
+        stream_lpeak = 0;
+        stream_rpeak = 0;
+        rec_lpeak = 0;
+        rec_rpeak = 0;
+        stream_lavg = 0;
+        stream_ravg = 0;
+        rec_lavg = 0;
+        rec_ravg = 0;
     }
 
-    if (vu_level_type == 0) {
-        p = stream_buf;
-    }
-    else {
-        p = record_buf;
-    }
     for (i = 0; i < framepacket_size; i += cfg.audio.channel) {
-        if (abs(p[i]) > lpeak) {
-            lpeak = abs(p[i]);
+        if (abs(stream_buf[i]) > stream_lpeak) {
+            stream_lpeak = abs(stream_buf[i]);
         }
-        if (abs(p[i + (cfg.audio.channel - 1)]) > rpeak) {
-            rpeak = abs(p[i + (cfg.audio.channel - 1)]);
+        if (abs(stream_buf[i + (cfg.audio.channel - 1)]) > stream_rpeak) {
+            stream_rpeak = abs(stream_buf[i + (cfg.audio.channel - 1)]);
+        }
+
+        if (abs(record_buf[i]) > rec_lpeak) {
+            rec_lpeak = abs(record_buf[i]);
+        }
+        if (abs(record_buf[i + (cfg.audio.channel - 1)]) > rec_rpeak) {
+            rec_rpeak = abs(record_buf[i + (cfg.audio.channel - 1)]);
         }
     }
 
-    float mean_peak = lpeak / 2 + rpeak / 2;
-    float mean_peak_dB = 20 * log10(mean_peak);
+    float mean_stream_peak = stream_lpeak / 2 + stream_rpeak / 2;
+    float mean_stream_peak_dB = 20 * log10(mean_stream_peak);
 
-    if (mean_peak_dB < -cfg.audio.silence_level) {
+    if (mean_stream_peak_dB < -cfg.audio.silence_level) {
         silence_detected = true;
     }
     else {
         silence_detected = false;
     }
 
-    if (mean_peak_dB > -cfg.audio.signal_level) {
+    if (mean_stream_peak_dB > -cfg.audio.signal_level) {
         signal_detected = true;
     }
     else {
         signal_detected = false;
     }
 
-    lavg = (decay * lpeak) + (1.0 - decay) * lavg;
-    ravg = (decay * rpeak) + (1.0 - decay) * ravg;
+    stream_lavg = (decay * stream_lpeak) + (1.0 - decay) * stream_lavg;
+    stream_ravg = (decay * stream_rpeak) + (1.0 - decay) * stream_ravg;
+    rec_lavg = (decay * rec_lpeak) + (1.0 - decay) * rec_lavg;
+    rec_ravg = (decay * rec_rpeak) + (1.0 - decay) * rec_ravg;
 
     // Update the vu meter UI only every second call of this function.
     // This reduces the CPU usage without not missing any samples
     if (call_cnt == 1) {
-        vu_meter(lavg, ravg);
+        if (vu_level_type == SND_STREAM) {
+            vu_meter(stream_lavg, stream_ravg, stream_lpeak, stream_rpeak);
+        }
+        else {
+            vu_meter(rec_lavg, rec_ravg, rec_lpeak, rec_rpeak);
+        }
 
-        if (cfg.dsp.compressor == 1) {
-            if (streaming_dsp->is_compressing == true) {
-                fl_g->LED_comp_threshold->set_state(LED::LED_ON);
-            }
-            else {
-                fl_g->LED_comp_threshold->set_state(LED::LED_OFF);
-            }
+        if (cfg.dsp.compressor_stream == 1) {
+            fl_g->LED_comp_threshold->set_state(streaming_dsp->is_compressing == true ? LED::LED_ON : LED::LED_OFF);
+        }
+        else if (cfg.dsp.compressor_rec == 1) {
+            fl_g->LED_comp_threshold->set_state(recording_dsp->is_compressing == true ? LED::LED_ON : LED::LED_OFF);
         }
 
         call_cnt = 0;
     }
     else {
         call_cnt++;
-        lpeak = 0;
-        rpeak = 0;
+        stream_lpeak = 0;
+        stream_rpeak = 0;
+        rec_lpeak = 0;
+        rec_rpeak = 0;
     }
 
     pa_new_frames = 0;
 }
 
-void snd_free_device_list(void)
+void snd_free_device_list(snd_dev_t **dev_list, int dev_count)
 {
-    if (cfg.audio.dev_count == 0) {
+    if (dev_count == 0) {
         return;
     }
 
     for (int i = 0; i < SND_MAX_DEVICES; i++) {
-        if (i < cfg.audio.dev_count) {
-            free(cfg.audio.pcm_list[i]->name);
+        if (i < dev_count) {
+            free(dev_list[i]->name);
         }
 
-        free(cfg.audio.pcm_list[i]);
+        free(dev_list[i]);
     }
-    free(cfg.audio.pcm_list);
+    free(dev_list);
 }
 
 snd_dev_t **snd_get_devices(int *dev_count)
@@ -1277,9 +1380,13 @@ snd_dev_t **snd_get_devices(int *dev_count)
             }
         }
 
-        // We need to escape every '/' in the device name
-        // otherwise FLTK will add a submenu for every '/' in the dev list
+        // Replace  characters from the device name that have a special meaning to FLTK
+        strrpl(&dev_list[dev_num]->name, (char *)"\\", (char *)" ", MODE_ALL); // Must come first
         strrpl(&dev_list[dev_num]->name, (char *)"/", (char *)"\\/", MODE_ALL);
+        strrpl(&dev_list[dev_num]->name, (char *)"|", (char *)" ", MODE_ALL);
+        strrpl(&dev_list[dev_num]->name, (char *)"\t", (char *)" ", MODE_ALL);
+        strrpl(&dev_list[dev_num]->name, (char *)"_", (char *)" ", MODE_ALL);
+        strrpl(&dev_list[dev_num]->name, (char *)"&", (char *)"+", MODE_ALL);
 
         dev_num++;
     } // for(i = 0; i < devcount && i < 100; i++)
@@ -1289,11 +1396,46 @@ snd_dev_t **snd_get_devices(int *dev_count)
     return dev_list;
 }
 
-/*    srconv_stream.end_of_input = 0;
-       srconv_stream.src_ratio = (float)samplerate_out / cfg.audio.samplerate;
-       srconv_stream.input_frames = frameCount;
-       srconv_stream.output_frames = frameCount * cfg.audio.channel * (srconv_stream.src_ratio + 1) * sizeof(float);
-   */
+void snd_print_devices(void)
+{
+    PaError err;
+    const PaDeviceInfo *di;
+
+    if ((err = Pa_Initialize()) != paNoError) {
+        printf("PortAudio init failed:\n%s\n", Pa_GetErrorText(err));
+        return;
+    }
+
+    int dev_count;
+    snd_dev_t **dev_list;
+    dev_list = snd_get_devices(&dev_count);
+
+    if (dev_count == 0) {
+        printf("No input audio device available\n");
+        return;
+    }
+
+    for (int i = 0; i < dev_count; i++) {
+        printf("Device ID: %d\n", i);
+        printf("Device name: %s\n", dev_list[i]->name);
+        printf("Number of channels: %d\n", dev_list[i]->num_of_channels);
+        printf("Supported sample rates: ");
+        for (int j = 0; j < dev_list[i]->num_of_sr; j++) {
+            if (j < dev_list[i]->num_of_sr - 1) {
+                printf("%d Hz, ", dev_list[i]->sr_list[j]);
+            }
+            else {
+                printf("%d Hz", dev_list[i]->sr_list[j]);
+            }
+        }
+
+        printf("\n\n");
+    }
+
+    snd_free_device_list(dev_list, dev_count);
+
+    Pa_Terminate();
+}
 
 void snd_reset_samplerate_conv(int rec_or_stream)
 {
@@ -1379,13 +1521,13 @@ void snd_close_streams(void)
     int stream2_is_active = Pa_IsStreamActive(stream2);
 
     if (stream2_is_active == 1) {
-        Pa_StopStream(stream2);
+        Pa_AbortStream(stream2);
         Pa_CloseStream(stream2);
     }
 
     if (stream_is_active == 1) {
         snd_stop_mixer_thread();
-        Pa_StopStream(stream);
+        Pa_AbortStream(stream);
         Pa_CloseStream(stream);
         g_stop_vu_meter_timer = 1;
         while (g_vu_meter_timer_is_active == 1) {
