@@ -12,12 +12,13 @@
 #include <unistd.h>
 #include <curl/curl.h>
 
-//#include "util.h"
 #include "url.h"
 #include "fl_funcs.h" // write_log()
+#include "config.h"   // VERSION
 
 int write_logfile = 0;
 int print_log = 0;
+int curl_initialized = 0;
 static char log_buf[100 * 1024];
 
 struct MemoryStruct {
@@ -181,8 +182,21 @@ static size_t callback_for_curl(void *contents, size_t size, size_t nmemb, void
     return realsize;
 }
 
+void url_init_curl(void)
+{
+    if (curl_initialized == 1) {
+        return;
+    }
+    curl_global_init(CURL_GLOBAL_DEFAULT);
+    curl_initialized = 1;
+}
+
 uint32_t url_post_json(const char *url, char *post_data, char *answer, uint32_t max_answer_size)
 {
+    if (curl_initialized == 0) {
+        url_init_curl();
+    }
+
     struct MemoryStruct chunk;
 
     chunk.memory = (char *)malloc(1); /* will be grown as needed by the realloc */
@@ -199,8 +213,6 @@ uint32_t url_post_json(const char *url, char *post_data, char *answer, uint32_t
     headers = curl_slist_append(headers, "Content-Type: application/json");
     headers = curl_slist_append(headers, "charset: utf-8");
 
-    curl_global_init(CURL_GLOBAL_DEFAULT);
-
     curl = curl_easy_init();
     if (curl) {
         if (print_log || write_logfile) {
@@ -231,7 +243,7 @@ uint32_t url_post_json(const char *url, char *post_data, char *answer, uint32_t
             long response_code;
             curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &response_code);
 
-            if (chunk.size + 1 > max_answer_size) {
+            if (chunk.size > max_answer_size) {
                 printf("%s\n", "buffer too small");
                 chunk.size = 0;
             }
@@ -255,12 +267,15 @@ uint32_t url_post_json(const char *url, char *post_data, char *answer, uint32_t
 
     free(chunk.memory);
 
-    curl_global_cleanup();
     return chunk.size;
 }
 
-uint32_t url_post(const char *url, char *post_data, char *answer, uint32_t max_answer_size)
+uint32_t url_post_sdp(const char *url, const char *bearer_token, char *post_data, char *answer, uint32_t max_answer_size)
 {
+    if (curl_initialized == 0) {
+        url_init_curl();
+    }
+
     struct MemoryStruct chunk;
 
     chunk.memory = (char *)malloc(1); /* will be grown as needed by the realloc */
@@ -272,7 +287,91 @@ uint32_t url_post(const char *url, char *post_data, char *answer, uint32_t max_a
     struct data config;
     config.trace_ascii = 1; /* enable ascii tracing for debug */
 
-    curl_global_init(CURL_GLOBAL_DEFAULT);
+    struct curl_slist *headers = NULL;
+    headers = curl_slist_append(headers, "Accept: application/sdp");
+    headers = curl_slist_append(headers, "Content-Type: application/sdp");
+    headers = curl_slist_append(headers, "Charset: utf-8");
+    if (bearer_token) {
+        char auth_header[256];
+        snprintf(auth_header, sizeof(auth_header), "Authorization: Bearer %s", bearer_token);
+        headers = curl_slist_append(headers, auth_header);
+    }
+
+    curl = curl_easy_init();
+    if (curl) {
+        char user_agent[32];
+        snprintf(user_agent, sizeof(user_agent), "butt %s", VERSION);
+        if (print_log || write_logfile) {
+            curl_easy_setopt(curl, CURLOPT_DEBUGFUNCTION, my_trace);
+            curl_easy_setopt(curl, CURLOPT_DEBUGDATA, &config);
+            curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L);
+        }
+
+        curl_easy_setopt(curl, CURLOPT_URL, url);
+        curl_easy_setopt(curl, CURLOPT_CUSTOMREQUEST, "POST");
+        curl_easy_setopt(curl, CURLOPT_HTTPHEADER, headers);
+        curl_easy_setopt(curl, CURLOPT_POSTFIELDS, post_data);
+        curl_easy_setopt(curl, CURLOPT_USERAGENT, user_agent);
+        curl_easy_setopt(curl, CURLOPT_TIMEOUT, 7L);
+        curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, 0L);
+        curl_easy_setopt(curl, CURLOPT_SSL_VERIFYHOST, 0L);
+        curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1L);
+        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, callback_for_curl);
+        curl_easy_setopt(curl, CURLOPT_WRITEDATA, (void *)&chunk);
+
+        /* Perform the request, res will get the return code */
+        res = curl_easy_perform(curl);
+        if (res != CURLE_OK) {
+            printf("URL request failed: %s\n", curl_easy_strerror(res));
+            chunk.size = 0;
+        }
+        else {
+            long response_code;
+            curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &response_code);
+
+            if (chunk.size > max_answer_size) {
+                printf("%s\n", "buffer too small");
+                chunk.size = 0;
+            }
+            /*     else if (response_code != 200) {
+                     printf("html response = %ld\n", response_code);
+                     chunk.size = 0;
+                 }
+                 else if (strstr(chunk.memory, "Invalid resource")) { // Shoutcast v1 response with http_code = 200 but "Invalid ressource"
+                                                                      // if a wrong url is requested
+                     printf("%s\n", "Invalid resource");
+                     chunk.size = 0;
+                 }*/
+            else {
+                memcpy(answer, chunk.memory, chunk.size);
+                answer[chunk.size] = '\0';
+            }
+        }
+
+        curl_easy_cleanup(curl);
+    }
+
+    free(chunk.memory);
+
+    return chunk.size;
+}
+
+uint32_t url_post(const char *url, char *post_data, char *answer, uint32_t max_answer_size)
+{
+    if (curl_initialized == 0) {
+        url_init_curl();
+    }
+
+    struct MemoryStruct chunk;
+
+    chunk.memory = (char *)malloc(1); /* will be grown as needed by the realloc */
+    chunk.size = 0;                   /* no data at this point */
+
+    CURL *curl;
+    CURLcode res;
+
+    struct data config;
+    config.trace_ascii = 1; /* enable ascii tracing for debug */
 
     curl = curl_easy_init();
     if (curl) {
@@ -306,7 +405,7 @@ uint32_t url_post(const char *url, char *post_data, char *answer, uint32_t max_a
             long response_code;
             curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &response_code);
 
-            if (chunk.size + 1 > max_answer_size) {
+            if (chunk.size > max_answer_size) {
                 printf("%s\n", "buffer too small");
                 write_log("url_post: buffer too small");
                 chunk.size = 0;
@@ -331,12 +430,15 @@ uint32_t url_post(const char *url, char *post_data, char *answer, uint32_t max_a
 
     free(chunk.memory);
 
-    curl_global_cleanup();
     return chunk.size;
 }
 
 uint32_t url_get(const char *url, const char *custom_hdr, char *answer, uint32_t data_size)
 {
+    if (curl_initialized == 0) {
+        url_init_curl();
+    }
+
     struct MemoryStruct chunk;
 
     chunk.memory = (char *)malloc(1); /* will be grown as needed by the realloc */
@@ -353,11 +455,11 @@ uint32_t url_get(const char *url, const char *custom_hdr, char *answer, uint32_t
         hdr = curl_slist_append(hdr, custom_hdr);
     }
 
-    curl_global_init(CURL_GLOBAL_DEFAULT);
-
     curl = curl_easy_init();
 
     if (curl) {
+        char user_agent[32];
+        snprintf(user_agent, sizeof(user_agent), "butt %s", VERSION);
         if (print_log || write_logfile) {
             curl_easy_setopt(curl, CURLOPT_DEBUGFUNCTION, my_trace);
             curl_easy_setopt(curl, CURLOPT_DEBUGDATA, &config);
@@ -365,8 +467,8 @@ uint32_t url_get(const char *url, const char *custom_hdr, char *answer, uint32_t
         }
 
         curl_easy_setopt(curl, CURLOPT_URL, url);
-        curl_easy_setopt(curl, CURLOPT_USERAGENT, "butt");
-        curl_easy_setopt(curl, CURLOPT_TIMEOUT, 7L);
+        curl_easy_setopt(curl, CURLOPT_USERAGENT, user_agent);
+        curl_easy_setopt(curl, CURLOPT_TIMEOUT, 3L);
         curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, 0L);
         curl_easy_setopt(curl, CURLOPT_SSL_VERIFYHOST, 0L);
         curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1L);
@@ -381,7 +483,7 @@ uint32_t url_get(const char *url, const char *custom_hdr, char *answer, uint32_t
         res = curl_easy_perform(curl);
         if (res != CURLE_OK) {
             printf("URL request failed: %s\n", curl_easy_strerror(res));
-            write_log("url_post: URL get failed: ");
+            write_log("url_get: URL get failed: ");
             write_log(curl_easy_strerror(res));
             write_log("\n");
             chunk.size = 0;
@@ -390,9 +492,9 @@ uint32_t url_get(const char *url, const char *custom_hdr, char *answer, uint32_t
             long response_code;
             curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &response_code);
 
-            if (chunk.size + 1 > data_size) {
+            if (chunk.size > data_size) {
                 printf("%s\n", "buffer too small");
-                write_log("url_post: buffer too small");
+                write_log("url_get: buffer too small");
                 chunk.size = 0;
             }
             else {
@@ -406,13 +508,16 @@ uint32_t url_get(const char *url, const char *custom_hdr, char *answer, uint32_t
 
     free(chunk.memory);
 
-    curl_global_cleanup();
     return chunk.size;
 }
 
 // Same as url_get() with a few icecast/shoutcast specific changes
 uint32_t url_get_listener_count(const char *url, char *data, uint32_t data_size)
 {
+    if (curl_initialized == 0) {
+        url_init_curl();
+    }
+
     struct MemoryStruct chunk;
 
     chunk.memory = (char *)malloc(1); /* will be grown as needed by the realloc */
@@ -421,8 +526,6 @@ uint32_t url_get_listener_count(const char *url, char *data, uint32_t data_size)
     CURL *curl;
     CURLcode res;
 
-    curl_global_init(CURL_GLOBAL_DEFAULT);
-
     curl = curl_easy_init();
     if (curl) {
         curl_easy_setopt(curl, CURLOPT_URL, url);
@@ -444,7 +547,7 @@ uint32_t url_get_listener_count(const char *url, char *data, uint32_t data_size)
             write_log("\n");
             chunk.size = 0;
         }
-        else {
+        else if (chunk.size > 0) {
             long response_code;
             curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &response_code);
 
@@ -472,6 +575,10 @@ uint32_t url_get_listener_count(const char *url, char *data, uint32_t data_size)
 
     free(chunk.memory);
 
-    curl_global_cleanup();
     return chunk.size;
 }
+
+void url_cleanup_curl(void)
+{
+    curl_global_cleanup();
+}
