@@ -27,6 +27,8 @@ struct vorbis_enc {
     vorbis_block vb;
     vorbis_dsp_state vd; /* central working state for the packet->PCM decoder */
 
+    int header_written;
+
     int bitrate;
     int samplerate;
     int channel;
@@ -54,7 +56,6 @@ int vorbis_enc_reinit(vorbis_enc *vorbis);
 void vorbis_enc_write_header(vorbis_enc *vorbis);
 void vorbis_enc_close(vorbis_enc *vorbis);
 
-void vorbis_enc_new_song(vorbis_enc *vorbis, char *song);
 int vorbis_enc_get_samplerate(vorbis_enc *vorbis);
 
 #endif
